#include <math.h>

/* This routine performs a Lorentz boost on the four vector */
/* contained in "vector". The three vector "beta"  is the   */
/* magnitude and direction of the frame which to boost into */


int boost(float *beta,float *vector)
{
   int i,j;
   float bmat[4][4];
   float beta_mag,betasq;
   float gamma;
   float v[4];
   
   /* setup some constants */
   beta_mag = beta[0]*beta[0] + beta[1]*beta[1] + beta[2]*beta[2];
   beta_mag = sqrt((double)beta_mag);
   betasq   = beta_mag*beta_mag;
   gamma    = 1.0/sqrt(1.0-betasq);
   
   if(betasq == 0.0)return 0;
   
   /* create transformation matrix */
   bmat[0][0] = gamma;
   bmat[0][1] = -beta[0]*gamma;
   bmat[0][2] = -beta[1]*gamma;
   bmat[0][3] = -beta[2]*gamma;

   bmat[1][0] = -beta[0]*gamma;
   bmat[1][1] = 1.0+((gamma-1.0)*beta[0]*beta[0])/betasq;
   bmat[1][2] =     ((gamma-1.0)*beta[0]*beta[1])/betasq;
   bmat[1][3] =     ((gamma-1.0)*beta[0]*beta[2])/betasq;
   
   bmat[2][0] = -beta[1]*gamma;
   bmat[2][1] =     ((gamma-1.0)*beta[1]*beta[0])/betasq;
   bmat[2][2] = 1.0+((gamma-1.0)*beta[1]*beta[1])/betasq;
   bmat[2][3] =     ((gamma-1.0)*beta[1]*beta[2])/betasq;
   
   bmat[3][0] = -beta[2]*gamma;
   bmat[3][1] =     ((gamma-1.0)*beta[2]*beta[0])/betasq;
   bmat[3][2] =     ((gamma-1.0)*beta[2]*beta[1])/betasq;
   bmat[3][3] = 1.0+((gamma-1.0)*beta[2]*beta[2])/betasq;  
   
   /* multiply vector by transformation matrix */
   v[0]=vector[0];
   v[1]=vector[1];
   v[2]=vector[2];
   v[3]=vector[3];
   
   for(j=0;j<4;j++){
      vector[j]=0.0;
      for(i=0;i<4;i++){
         vector[j]+=v[i]*bmat[j][i];
      }
   }

   return 0;
}
