


#include "psim.h"

/**************************************************************/
/* This returns the energy of a brehmstralung photon given an */
/* incident electron energy.                                  */
/* Fractional limits can be placed on the values returned by  */
/* setting EGFRAC(1) and EGFRAC(2) in the PHBMEN common block.*/
/**************************************************************/

float brehmstrahlung(float E)
{
   double Emin;
   double Emax;
   double s,brehmE;
   float *egamak;
   
   /* make lower limit 100 MeV and no upper limit cut on electron energy */
   phbmen_.egfrac[0]=0.100/E;
   phbmen_.egfrac[1]=1.0;
   
   egamak=&brmst1_.egamak;
   Emin=(double)E*(double)phbmen_.egfrac[0];
   Emax=(double)E*(double)phbmen_.egfrac[1];
      
   if(Emin<1.0E-6)Emin=1.0E-6; /* force minimum to be at least 1keV */
   if(Emax<=Emin || Emax>(double)E)Emax=(double)E;
      
   s=(double)(RAND_MAX&random())/(double)RAND_MAX;
   
   brehmE = Emin * pow( Emax/Emin,s);
   *egamak=(float)brehmE;
   
   return (float)brehmE;
}


