
extern "C" {
#include "psim.h"
}

#include "operators.h"
#include "eta.h"
//#include <primex.h>



#define MESON_ETA "eta"
#define MESON_MASS_ETA 0.54730                      // in GeV
#define MESON_ETA_PRIME "eta prime"
#define MESON_MASS_ETA_PRIME 0.95777                // in GeV
#define TARGET_A 12.0
#define TARGET_Z 6.0

char *MESON=MESON_ETA;
float MESON_MASS=MESON_MASS_ETA;



extern "C" {

int eta(float E);
int eta_He4(float E);
int eta_C12(float E);
int eta_prime_C12(float E);
float calc_q(float theta, float m, float M, float k);

void gsvert_(float *VERT,int *zero,int *zero,float *UBUF,int *zero,int *NVTX);
void gskine_(float *PLAB,int *IPART,int *NVTX,float *UBUF,int *zero,int *NT);
}

// Routine declarations
int GenerateEta(float E,int flags,float *theta,float *phi,float *momentum);
float RANDOM(void);
Vect4* DecayEta(float theta,float phi,float p,int *N,int **types,int *decay_mech);
Vect3 Rotate(Vect3 p,float theta,float phi);
Vect4 Boost(Vect4 v,Vect3 b);
void find_theta_phi(Vect3 v, float *theta,float *phi_ret);
void find_theta_phi(Vect4 v, float *theta,float *phi_ret);


float TARGET_MASS;
cross_table_ptr_t *eta_cross=eta_cross_He4;

int eta_He4(float E)
{
   eta_cross=eta_cross_He4;
   MESON_MASS=MESON_MASS_ETA;
   MESON=MESON_ETA;
   return eta(E);
}

int eta_C12(float E)
{
   eta_cross=eta_cross_C12;
   MESON_MASS=MESON_MASS_ETA;
   MESON=MESON_ETA;
   return eta(E);
}


int eta(float E)
{
	int i;
   int mech,decay_mech;
   int *types;
   float theta,phi,p;
   float theta1,phi1,theta2,phi2;
   float p1,p2;
   Vect4 *decay_parts;
   int N;
   float VERT[3]={0.0,0.0,0.0};
   float PLAB[3]={0.0,0.0,0.0};
   float UBUF;
   float beam_size;
   int IPART;
   int NVTX,NT,HID;
   float thetas[10];

   if(E<5.6)return -1;

   // 4He= 3.728 (this is from (4.002603amu)(931.502 MeV/amu) )
   // 12C=11.178 (this is from (12amu)(931.502 MeV/amu) )
	TARGET_MASS=11.178;

   // Generate an eta
	mech=GenerateEta(E,PRIM_CROSS|NUCL_CROSS|INTER_CROSS,&theta,&phi,&p);
   initial_4vect.E=sqrt(pow(p,2.0)+pow(MESON_MASS,2.0));
   initial_4vect.x=p*sin(theta)*cos(phi);
   initial_4vect.y=p*sin(theta)*sin(phi);
   initial_4vect.z=p*cos(theta);
   
   // The bremstrahlung generator can generate events below the
   // eta production threshold. If this happens, the eta momentum
   // will be NaN.
   if(isnan(p))return -1;
   if(E<MESON_MASS)
      printf("E<MESON_MASS!!! p=%f E=%f theta=%f phi=%f MESON_MASS=%f\n"
         ,p,E,theta,phi,MESON_MASS);

   // Decay the eta
   decay_parts=DecayEta(theta,phi,p,&N,&types,&decay_mech);
   if(N!=2 && N!=3){cerr<<"Error in"<<__FILE__<<" at line "<<__LINE__<<"\n";return -2;}
   if(decay_parts[0].Mass()!=0.0
      || decay_parts[1].Mass()!=0.0){/*return -5;*/}

   event_subtype=mech;
   event_decay_mech=decay_mech;


/*
   find_theta_phi(decay_parts[0], &theta1, &phi1);
   find_theta_phi(decay_parts[1], &theta2, &phi2);
   p1=decay_parts[0].p();
   p2=decay_parts[1].p();
*/

   /* determine initial position of eta and */
   /* randomly place in a 250 micron radius */
   /* at the center of target, evenly       */
   /* distributed along z                   */
   beam_size=RANDOM()*250.0E-4;
   phi=RANDOM()*2.0*PI;
   VERT[0] = beam_size*cos(phi);
   VERT[1] = beam_size*sin(phi);
   VERT[2] = thargt_.thrpos[2] + ((RANDOM()*50.0)-20.0);
   gsvert_(VERT,&zero,&zero,&UBUF,&zero,&NVTX);
   initial_position.x=VERT[0];
   initial_position.y=VERT[1];
   initial_position.z=VERT[2];
   
   // Define the resulting decay products to GEANT
   for(i=0;i<N;i++){
      IPART=types[i];
      find_theta_phi(decay_parts[i], &theta, &phi);
      thetas[i]=theta;
      theta*=DEG2RAD;
      phi  *=DEG2RAD;
      p=decay_parts[i].p();
      PLAB[0]=p*cos(phi)*sin(theta);
      PLAB[1]=p*sin(phi)*sin(theta);
      PLAB[2]=p*cos(theta);

      if(!flags[generate_only])gskine_(PLAB,&IPART,&NVTX,&UBUF,&zero,&NT);
   }
   if(decay_mech==0)hf2(5003,thetas[0],thetas[1],1.0);

   return 0;
}

/*=========================================================*/
/* Random number generator. This returns a random number
/* between 0 and 1.
/*=========================================================*/
float RANDOM(void){return (float)random()/(float)RAND_MAX;}



int GenerateEta(float E,int flags,float *theta,float *phi,float *momentum)
{
	int i,j;
	int index,mech;
	float sum,sum_total,sum_prim,sum_nucl,sum_inter;
	float r,*c;
	int who;
	float theta1,theta2;

	/* first, find energy tables */
	for(i=1;i<NUM_ETA_TABLES;i++){
		if(eta_cross[i].E>E)break;
	}
	if(i>=NUM_ETA_TABLES)return -1;

	index=i;

	/* determine by which mechansim to generate this event */
	sum=0.0;
	sum_total=eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].total;
	sum_prim =eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].prim;
	sum_nucl =eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].nucl;
	sum_inter=eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].inter;
	if(flags&TOTAL_CROSS)sum=sum_total=sum_total+sum; else sum_total=-1.0;
	if(flags&PRIM_CROSS )sum=sum_prim =sum_prim +sum; else sum_prim =-1.0;
	if(flags&NUCL_CROSS )sum=sum_nucl =sum_nucl +sum; else sum_nucl =-1.0;
	if(flags&INTER_CROSS)sum=sum_inter=sum_inter+sum; else sum_inter=-1.0;

	if(sum<=0.0)return -2;

	r=RANDOM();
	if(sum_inter/sum > r)mech=INTER_CROSS;
	if(sum_nucl /sum > r)mech=NUCL_CROSS;
	if(sum_prim /sum > r)mech=PRIM_CROSS;
	if(sum_total/sum > r)mech=TOTAL_CROSS;

	switch(mech){
		case TOTAL_CROSS:
			who=(long)&eta_cross[index].t[i].total-(long)&eta_cross[index].t[i];
			sum=eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].total;
			break;
		case PRIM_CROSS:
			who=(long)&eta_cross[index].t[i].prim -(long)&eta_cross[index].t[i];
			sum=eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].prim;
			break;
		case NUCL_CROSS:
			who=(long)&eta_cross[index].t[i].nucl -(long)&eta_cross[index].t[i];
			sum=eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].nucl;
			break;
		case INTER_CROSS:
			who=(long)&eta_cross[index].t[i].inter-(long)&eta_cross[index].t[i];
			sum=eta_cross[index].t[NUM_ETA_TABLE_ENTRIES-1].inter;
			break;
	}
	who/=sizeof(float);

	r=RANDOM();
	
	/*---------- determine theta of eta at index -----------*/
	for(i=1;i<NUM_ETA_TABLE_ENTRIES;i++){
		c=(float*)&eta_cross[index].t[i];
		if((c[who]/sum)>r)break;
	}
	if(i>=NUM_ETA_TABLE_ENTRIES)return -3;
	/* interpolate thetas */
	theta1=eta_cross[index].t[i].theta-eta_cross[index].t[i-1].theta;
	theta1*=(c[who]/sum) - r;
	theta1/=c[who] - ((float*)&eta_cross[index].t[i-1])[who];
	theta1=eta_cross[index].t[i].theta - theta1;

	/*--------- determine theta of eta at index-1 ----------*/
	switch(mech){
		case TOTAL_CROSS:
			sum=eta_cross[index-1].t[NUM_ETA_TABLE_ENTRIES-1].total;
			break;
		case PRIM_CROSS:
			sum=eta_cross[index-1].t[NUM_ETA_TABLE_ENTRIES-1].prim;
			break;
		case NUCL_CROSS:
			sum=eta_cross[index-1].t[NUM_ETA_TABLE_ENTRIES-1].nucl;
			break;
		case INTER_CROSS:
			sum=eta_cross[index-1].t[NUM_ETA_TABLE_ENTRIES-1].inter;
			break;
	}
	for(i=1;i<NUM_ETA_TABLE_ENTRIES;i++){
		c=(float*)&eta_cross[index-1].t[i];
		if((c[who]/sum)>r)break;
	}
	if(i>=NUM_ETA_TABLE_ENTRIES)return -4;
	/* interpolate thetas */
	theta2=eta_cross[index-1].t[i].theta-eta_cross[index-1].t[i-1].theta;
	theta2*=(c[who]/sum) - r;
	theta2/=c[who] - ((float*)&eta_cross[index-1].t[i-1])[who];
	theta2=eta_cross[index-1].t[i].theta - theta2;

	/*-------- Interpolate between the two thetas ----------*/
	*theta=theta1-theta2;
	(*theta)*=E-eta_cross[index-1].E;
	(*theta)/=eta_cross[index].E-eta_cross[index-1].E;
	(*theta)+=theta2;

	// convert from mrad to rad
	(*theta)/=1000.0;

	/*-------- phi is evenly distributed ---------*/
	*phi=2.0*acos(-1.0)*RANDOM();

	/*-------- calculate magnitude of momentum -------*/
	*momentum=calc_q(*theta, MESON_MASS, TARGET_MASS, E);

	return mech;
}

// =============================================================
// Decay the meson. Right now, there is only one decay channel
// supported. The meson->2photon channel
// =============================================================
Vect4* DecayEta(float theta,float phi,float p,int *Num,int **types,int *mech)
{
	int i,j,N;
	static Vect4 parts[32],tp;
   static int parttype[32];
	Vect3 beta;
	float E,b;
	float ph,th;
   float chan_weight[32];
   int Num_chans=0;
   float r;
   float PCM[3][4]={{0.0,0.0,0.0,0.0},{0.0,0.0,0.0,0.0},{0.0,0.0,0.0,0.0}};
   
	// Create a 3 vector for the beta of the meson in the lab frame
	E=sqrt(pow(MESON_MASS,2.0)+pow(p,2.0));
	b=p/E;
	beta.px=b*(sin(theta)*cos(phi));
	beta.py=b*(sin(theta)*sin(phi));
	beta.pz=b*(cos(theta));

   // Deterimine decay channel
   chan_weight[Num_chans++]=0.3921; // 2 gamma
   chan_weight[Num_chans++]=0.3220; // 3 pi0
   chan_weight[Num_chans++]=0.2310; // pi+, pi-, pi0
   chan_weight[Num_chans++]=0.0477; // pi+, pi-, gamma
   for(i=1;i<Num_chans;i++)chan_weight[i]+=chan_weight[i-1];
   r=chan_weight[Num_chans-1]*RANDOM();
   for(i=Num_chans-1;i>=0;i--)
      if(chan_weight[i]>r)*mech=i;

   // Create decay products in rest frame of decaying meson
   N=0;
   switch(*mech){
      case 0: // 2gamma
         gdeca2(MESON_MASS ,0.0 ,0.0 ,(float**)PCM);
         parttype[N++]=ptype_gamma;
         parttype[N++]=ptype_gamma;
         break;
      case 1: // 3 pi0
         gdeca3(MESON_MASS ,PI_ZERO_MASS ,PI_ZERO_MASS ,PI_ZERO_MASS ,(float**)PCM);
         parttype[N++]=ptype_pion_zero;
         parttype[N++]=ptype_pion_zero;
         parttype[N++]=ptype_pion_zero;
         break;
      case 2: // pi+, pi-, pi0
         gdeca3(MESON_MASS ,PI_CHARGED_MASS ,PI_CHARGED_MASS ,PI_ZERO_MASS ,(float**)PCM);
         parttype[N++]=ptype_pion_plus;
         parttype[N++]=ptype_pion_minus;
         parttype[N++]=ptype_pion_zero;
         break;
      case 3: // pi+, pi-, gamma
         gdeca3(MESON_MASS ,PI_CHARGED_MASS ,PI_CHARGED_MASS ,0.0 ,(float**)PCM);
         parttype[N++]=ptype_pion_plus;
         parttype[N++]=ptype_pion_minus;
         parttype[N++]=ptype_gamma;
         break;
      default:
         cerr<<"\nError in "<<__FILE__<<" near line "<<__LINE__<<"\n\n";
         cerr<<"*mech="<<*mech<<"  r="<<r<<"\n";
   }


	// Boost the decay products into the lab frame
	for(i=0;i<N;i++){
      parts[i].px=PCM[i][0];
      parts[i].py=PCM[i][1];
      parts[i].pz=PCM[i][2];
      parts[i].E =PCM[i][3];
      
		parts[i]=Boost(parts[i],beta);
		//parts[i]=Boost(parts[i],Vect3(0.0, 0.0, 0.0));
		if(fabs(1.0 - (parts[i].E/parts[i].p())) <1.0E-5)parts[i].E=parts[i].p();
	}

   *types=parttype;
	*Num=N;
	return parts;
}



Vect4 Boost(Vect4 v,Vect3 b)
{
	float theta,phi;
	float beta,gamma,pz;
	Vect3 p;
	static Vect4 bv;

	beta=b.p();
	gamma=1.0/sqrt(1.0-pow(beta,2.0));

	p.px=v.px;
	p.py=v.py;
	p.pz=v.pz;

	// Find theta,phi of b
	find_theta_phi(b, &theta, &phi);
	theta*=PI/180.0;
	phi*=PI/180.0;

	// Rotate vector so that beta is in z-direction
	p=Rotate(p, 0.0, -phi);
	p=Rotate(p, -theta, 0.0);

	// Boost along z-axis
	pz=p.pz;
	p.pz = (gamma*pz ) + (gamma*beta*v.E);
	bv.E  = (gamma*v.E) + (gamma*beta*pz );

	// Need to make sure we rotate back in the right order
	p=Rotate(p, theta  , 0.0);
	p=Rotate(p, 0.0 , phi );

	bv.px=p.px;
	bv.py=p.py;
	bv.pz=p.pz;
	
	return bv;
}

Vect3 Rotate(Vect3 p,float theta,float phi)
{
	Vect3 q;

	q=p;
	
	// Rotate by theta about Y
	q.px=(p.px*cos(theta)) + (p.pz*sin(theta));
	q.pz=(p.pz*cos(theta)) - (p.px*sin(theta));

	p=q;

	// Rotate by phi about Z
	q.px=(p.px*cos(phi)) + (p.py*sin(phi));
	q.py=(p.py*cos(phi)) - (p.px*sin(phi));

	return q;
}

//=====================================
// find theta and phi from 3 vector
//=====================================
void find_theta_phi(Vect4 v, float *theta,float *phi_ret)
{
	Vect3 w;
	w.px=v.px;  w.py=v.py;  w.pz=v.pz;
	find_theta_phi(w, theta, phi_ret);
}
void find_theta_phi(Vect3 v, float *theta,float *phi_ret)
{
   float phi=0.0,r;
   int sector;

   /* phi */
   r=sqrt((v.px*v.px) + (v.py*v.py));
   if(r!=0.0)phi = (180.0/PI)*acos(v.px/r);
   if(v.py<0.0)phi = 360.0-phi;
   while(phi<0.0)phi+=360.0;
   while(phi>360.0)phi-=360.0;
   *phi_ret=phi;

   /* theta */
   r=sqrt((v.px*v.px) + (v.py*v.py) + (v.pz*v.pz));
   if(r!=0.0)*theta = (180.0/PI)*acos(v.pz/r);
   while(*theta<0.0)*theta+=180.0;
   if(v.px==0.0 && v.py==0.0)*theta=0.0;


}




