

#include "psim.h"

/* pi0 decay */


#define Nbins 280
#define MAX_ANGLE 2.0
extern float pi0_cross_sections[280][5];
float pi0_theta_bins[Nbins];
float calc_q(float theta, float m, float M, float k);


/**********************************/
/* Initializer for pi0 generator. */
/**********************************/
void init_pi0(void)
{
   int i;
   float T=0.0;
   
   pi0_theta_bins[0]=0.0;
   for(i=1;i<Nbins;i++){
      pi0_theta_bins[i]=
         pi0_theta_bins[i-1]+pi0_cross_sections[i][1];
   }
   
   /* Normalize */
   for(i=1;i<Nbins;i++)pi0_theta_bins[i]/=pi0_theta_bins[Nbins-1];

}



/*******************************************************/
/* Generate a pi0. Geant will decay it into two gammas */
/*******************************************************/
int pi0(float E)
{
   float VERT[3]={0.0,0.0,0.0};
   float PLAB[3]={0.0,0.0,0.0};
   float UBUF;
   float beam_size;
   int IPART=ptype_pion_zero;
   int NVTX,NT,HID;
   float theta,phi,P,a;
   static int initialized=0;
   int i;
   
   if(!initialized){init_pi0();initialized=1;}
      
   /* Sample the total pi0 cross section as function of theta */
   do{
      a=ranl(0.0,1.0);
      for(i=1;i<Nbins-1;i++)if(pi0_theta_bins[i]>a)break;
      theta=a*(pi0_cross_sections[i][0]-pi0_cross_sections[i-1][0]);
      theta+=pi0_cross_sections[i-1][0];
   }while(theta<1.0E-10);
   
   /* Find pi0 momentum at this angle */
   /* (using 208Pb)                   */
   P=calc_q(theta, 0.1349764, 193.73, E);
   if(isnan(P))return -1;
   
   /* determine initial position and direction of pi0 */
   /* randomly place in a 250 micron radius at the center of target */
   beam_size=ranl(0.0,250.0E-4);
   phi=ranl(0.0,2.0*PI);
   VERT[0] = beam_size*cos(phi);
   VERT[1] = beam_size*sin(phi);
   VERT[2] = 0.0;
   gsvert_(VERT,&zero,&zero,&UBUF,&zero,&NVTX);
   initial_position.x=VERT[0];
   initial_position.y=VERT[1];
   initial_position.z=VERT[2];

   theta*=PI/180.0;
   phi=ranl(0.0,2.0*PI);
   PLAB[0]=P*cos(phi)*sin(theta);
   PLAB[1]=P*sin(phi)*sin(theta);
   PLAB[2]=P*cos(theta);

   if(!flags[generate_only])gskine_(PLAB,&IPART,&NVTX,&UBUF,&zero,&NT);
   initial_4vect.E=E;
   initial_4vect.x=PLAB[0];
   initial_4vect.y=PLAB[1];
   initial_4vect.z=PLAB[2];
   
   return 0;   
}

/*=========================================================*/
/* calculate magnitude of produced hadron's particle's 
/* final state momentum from 2-body kinematics
/*=========================================================*/
float calc_q(float theta, float m, float M, float k)
{
   float q;
   float a,b,c;
   float alpha,beta;
   float cos_th,sin_th;
   
   alpha = ( (m*m) + (2.0*k*M) )/(2.0*(k+M));
   beta  = k/(k+M);
   
   cos_th=cos(theta);
   sin_th=sin(theta);
   a = 1.0 - pow(beta*cos_th,2.0);
   b = -2.0*alpha*beta*cos_th;
   c = (m*m) - (alpha*alpha);
   
   q = -b + sqrt( (b*b) - (4.0*a*c) );
   q /= 2.0*a;
   
   return q;
}

/******************************************************/
/* The values in this table were generated using the
/* code under cvs control on the JLab CUE in
/* /group/primex/CVS . The package is in
/* programs/theory/cross-section
/******************************************************/
float pi0_cross_sections[280][5]={
	{0.0025, 133.498, 133.272, 0.000382009, 0.225635},  
	{0.0075, 868.264, 866.535, 0.00343757, 1.72591},  
	{0.0125, 1427.7, 1424, 0.009546, 3.68694},  
	{0.0175, 1545.03, 1539.64, 0.0187019, 5.36602},  
	{0.0225, 1417.28, 1410.65, 0.0308972, 6.60189},  
	{0.0275, 1217.64, 1210.12, 0.046121, 7.47075},  
	{0.0325, 1022.44, 1014.29, 0.06436, 8.07959},  
	{0.0375, 854.96, 846.363, 0.085598, 8.51158},  
	{0.0425, 717.84, 708.907, 0.109816, 8.82323},  
	{0.0475, 607.27, 598.081, 0.136993, 9.05169},  
	{0.0525, 518.249, 508.861, 0.167105, 9.22135},  
	{0.0575, 446.245, 436.697, 0.200125, 9.34849},  
	{0.0625, 387.572, 377.892, 0.236025, 9.44414},  
	{0.0675, 339.351, 329.56, 0.274772, 9.51598},  
	{0.0725, 299.371, 289.486, 0.316332, 9.5694},  
	{0.0775, 265.938, 255.969, 0.360668, 9.60833},  
	{0.0825, 237.748, 227.704, 0.407743, 9.6356},  
	{0.0875, 213.793, 203.682, 0.457513, 9.65336},  
	{0.0925, 193.29, 183.117, 0.509936, 9.66322},  
	{0.0975, 175.622, 165.39, 0.564965, 9.66643},  
	{0.1025, 160.301, 150.014, 0.622552, 9.66393},  
	{0.1075, 146.937, 136.598, 0.682645, 9.65651},  
	{0.1125, 135.218, 124.828, 0.745194, 9.64475},  
	{0.1175, 124.889, 114.45, 0.810142, 9.62915},  
	{0.1225, 115.742, 105.255, 0.877432, 9.6101},  
	{0.1275, 107.608, 97.0727, 0.947006, 9.58793},  
	{0.1325, 100.343, 89.7616, 1.0188, 9.56292},  
	{0.1375, 93.8317, 83.2036, 1.09276, 9.53528},  
	{0.1425, 87.974, 77.2999, 1.16881, 9.50521},  
	{0.1475, 82.687, 71.9672, 1.24689, 9.47288},  
	{0.1525, 77.9003, 67.135, 1.32693, 9.43841},  
	{0.1575, 73.5539, 62.7431, 1.40887, 9.40194},  
	{0.1625, 69.5964, 58.7402, 1.49262, 9.36358},  
	{0.1675, 65.9837, 55.0821, 1.57812, 9.32341},  
	{0.1725, 62.6776, 51.7308, 1.66529, 9.28152},  
	{0.1775, 59.6453, 48.6532, 1.75405, 9.23799},  
	{0.1825, 56.8579, 45.8207, 1.84434, 9.19288},  
	{0.1875, 54.2904, 43.2081, 1.93607, 9.14625},  
	{0.1925, 51.9209, 40.7935, 2.02916, 9.09817},  
	{0.1975, 49.7299, 38.5577, 2.12354, 9.04868},  
	{0.2025, 47.7005, 36.4836, 2.21911, 8.99784},  
	{0.2075, 45.8177, 34.5562, 2.3158, 8.94568},  
	{0.2125, 44.0679, 32.7621, 2.41353, 8.89226},  
	{0.2175, 42.4394, 31.0895, 2.51221, 8.83761},  
	{0.2225, 40.9215, 29.5279, 2.61175, 8.78178},  
	{0.2275, 39.5046, 28.0678, 2.71208, 8.72479},  
	{0.2325, 38.1804, 26.7006, 2.8131, 8.6667},  
	{0.2375, 36.9412, 25.4189, 2.91473, 8.60752},  
	{0.2425, 35.78, 24.2158, 3.01689, 8.5473},  
	{0.2475, 34.6906, 23.085, 3.11948, 8.48606},  
	{0.2525, 33.6674, 22.0211, 3.22242, 8.42385},  
	{0.2575, 32.7053, 21.019, 3.32562, 8.36069},  
	{0.2625, 31.7996, 20.074, 3.429, 8.29661},  
	{0.2675, 30.9462, 19.1821, 3.53246, 8.23164},  
	{0.2725, 30.1411, 18.3393, 3.63593, 8.16581},  
	{0.2775, 29.3809, 17.5424, 3.73931, 8.09916},  
	{0.2825, 28.6623, 16.7881, 3.84251, 8.03171},  
	{0.2875, 27.9824, 16.0735, 3.94546, 7.96349},  
	{0.2925, 27.3385, 15.3959, 4.04806, 7.89453},  
	{0.2975, 26.7281, 14.753, 4.15023, 7.82485},  
	{0.3025, 26.1488, 14.1425, 4.25189, 7.7545},  
	{0.3075, 25.5987, 13.5623, 4.35295, 7.68348},  
	{0.3125, 25.0757, 13.0105, 4.45333, 7.61184},  
	{0.3175, 24.578, 12.4854, 4.55294, 7.53959},  
	{0.3225, 24.1039, 11.9854, 4.65171, 7.46677},  
	{0.3275, 23.6519, 11.509, 4.74955, 7.39341},  
	{0.3325, 23.2206, 11.0547, 4.84638, 7.31952},  
	{0.3375, 22.8086, 10.6214, 4.94213, 7.24515},  
	{0.3425, 22.4147, 10.2077, 5.03672, 7.1703},  
	{0.3475, 22.0377, 9.81261, 5.13007, 7.09502},  
	{0.3525, 21.6765, 9.43508, 5.22211, 7.01933},  
	{0.3575, 21.3302, 9.07415, 5.31276, 6.94325},  
	{0.3625, 20.9977, 8.72892, 5.40195, 6.86682},  
	{0.3675, 20.6782, 8.39854, 5.48961, 6.79005},  
	{0.3725, 20.3709, 8.08224, 5.57568, 6.71297},  
	{0.3775, 20.075, 7.77928, 5.66008, 6.63561},  
	{0.3825, 19.7897, 7.48898, 5.74275, 6.558},  
	{0.3875, 19.5145, 7.21069, 5.82362, 6.48015},  
	{0.3925, 19.2486, 6.94383, 5.90264, 6.4021},  
	{0.3975, 18.9914, 6.68781, 5.97974, 6.32387},  
	{0.4025, 18.7425, 6.44212, 6.05486, 6.24549},  
	{0.4075, 18.5012, 6.20625, 6.12795, 6.16698},  
	{0.4125, 18.267, 5.97974, 6.19896, 6.08836},  
	{0.4175, 18.0396, 5.76214, 6.26782, 6.00966},  
	{0.4225, 17.8184, 5.55303, 6.33449, 5.9309},  
	{0.4275, 17.6031, 5.35204, 6.39892, 5.85212},  
	{0.4325, 17.3932, 5.15878, 6.46106, 5.77332},  
	{0.4375, 17.1883, 4.97291, 6.52088, 5.69453},  
	{0.4425, 16.9882, 4.7941, 6.57832, 5.61579},  
	{0.4475, 16.7925, 4.62203, 6.63334, 5.5371},  
	{0.4525, 16.6008, 4.45642, 6.68592, 5.4585},  
	{0.4575, 16.413, 4.29697, 6.73601, 5.38},  
	{0.4625, 16.2287, 4.14344, 6.78358, 5.30164},  
	{0.4675, 16.0476, 3.99556, 6.82859, 5.22342},  
	{0.4725, 15.8695, 3.85311, 6.87103, 5.14537},  
	{0.4775, 15.6942, 3.71585, 6.91086, 5.06751},  
	{0.4825, 15.5215, 3.58356, 6.94806, 4.98987},  
	{0.4875, 15.3511, 3.45606, 6.9826, 4.91246},  
	{0.4925, 15.1829, 3.33314, 7.01448, 4.83531},  
	{0.4975, 15.0167, 3.21462, 7.04366, 4.75843},  
	{0.5025, 14.8523, 3.10032, 7.07014, 4.68185},  
	{0.5075, 14.6896, 2.99008, 7.09391, 4.60557},  
	{0.5125, 14.5283, 2.88373, 7.11494, 4.52964},  
	{0.5175, 14.3684, 2.78114, 7.13324, 4.45405},  
	{0.5225, 14.2098, 2.68215, 7.14881, 4.37883},  
	{0.5275, 14.0522, 2.58662, 7.16162, 4.304},  
	{0.5325, 13.8957, 2.49443, 7.1717, 4.22957},  
	{0.5375, 13.74, 2.40544, 7.17903, 4.15557},  
	{0.5425, 13.5852, 2.31955, 7.18362, 4.082},  
	{0.5475, 13.431, 2.23662, 7.18548, 4.00889},  
	{0.5525, 13.2774, 2.15657, 7.18462, 3.93626},  
	{0.5575, 13.1244, 2.07927, 7.18104, 3.8641},  
	{0.5625, 12.9718, 2.00463, 7.17476, 3.79246},  
	{0.5675, 12.8197, 1.93255, 7.1658, 3.72132},  
	{0.5725, 12.6678, 1.86294, 7.15416, 3.65073},  
	{0.5775, 12.5163, 1.79572, 7.13988, 3.58067},  
	{0.5825, 12.3649, 1.73079, 7.12296, 3.51118},  
	{0.5875, 12.2138, 1.66808, 7.10344, 3.44226},  
	{0.5925, 12.0628, 1.60751, 7.08134, 3.37392},  
	{0.5975, 11.9119, 1.54901, 7.05669, 3.30618},  
	{0.6025, 11.7611, 1.49249, 7.02951, 3.23906},  
	{0.6075, 11.6103, 1.4379, 6.99984, 3.17255},  
	{0.6125, 11.4596, 1.38517, 6.96771, 3.10668},  
	{0.6175, 11.3088, 1.33423, 6.93316, 3.04145},  
	{0.6225, 11.1581, 1.28502, 6.89622, 2.97688},  
	{0.6275, 11.0074, 1.23749, 6.85693, 2.91297},  
	{0.6325, 10.8566, 1.19158, 6.81533, 2.84973},  
	{0.6375, 10.7059, 1.14722, 6.77146, 2.78718},  
	{0.6425, 10.5551, 1.10438, 6.72536, 2.72532},  
	{0.6475, 10.4042, 1.063, 6.67709, 2.66416},  
	{0.6525, 10.2534, 1.02303, 6.62668, 2.60371},  
	{0.6575, 10.1026, 0.984423, 6.57418, 2.54397},  
	{0.6625, 9.95174, 0.947136, 6.51965, 2.48495},  
	{0.6675, 9.80092, 0.911125, 6.46313, 2.42667},  
	{0.6725, 9.65014, 0.876347, 6.40467, 2.36912},  
	{0.6775, 9.49941, 0.842762, 6.34434, 2.31231},  
	{0.6825, 9.34875, 0.810332, 6.28217, 2.25625},  
	{0.6875, 9.19818, 0.779018, 6.21823, 2.20093},  
	{0.6925, 9.04773, 0.748784, 6.15257, 2.14638},  
	{0.6975, 8.89743, 0.719595, 6.08525, 2.09259},  
	{0.7025, 8.7473, 0.691416, 6.01633, 2.03956},  
	{0.7075, 8.59737, 0.664216, 5.94586, 1.98729},  
	{0.7125, 8.44767, 0.637962, 5.87391, 1.9358},  
	{0.7175, 8.29824, 0.612624, 5.80053, 1.88508},  
	{0.7225, 8.1491, 0.588173, 5.72579, 1.83514},  
	{0.7275, 8.0003, 0.564578, 5.64974, 1.78598},  
	{0.7325, 7.85187, 0.541814, 5.57246, 1.73759},  
	{0.7375, 7.70384, 0.519853, 5.49399, 1.68999},  
	{0.7425, 7.55625, 0.498669, 5.41442, 1.64317},  
	{0.7475, 7.40915, 0.478237, 5.33378, 1.59713},  
	{0.7525, 7.26256, 0.458533, 5.25216, 1.55187},  
	{0.7575, 7.11654, 0.439533, 5.16962, 1.50739},  
	{0.7625, 6.97112, 0.421216, 5.08621, 1.46369},  
	{0.7675, 6.82634, 0.403558, 5.00201, 1.42077},  
	{0.7725, 6.68225, 0.386539, 4.91707, 1.37864},  
	{0.7775, 6.53888, 0.370137, 4.83146, 1.33728},  
	{0.7825, 6.39627, 0.354334, 4.74525, 1.29669},  
	{0.7875, 6.25448, 0.339109, 4.65849, 1.25688},  
	{0.7925, 6.11354, 0.324445, 4.57126, 1.21783},  
	{0.7975, 5.97349, 0.310321, 4.48361, 1.17956},  
	{0.8025, 5.83438, 0.296722, 4.39561, 1.14205},  
	{0.8075, 5.69625, 0.283631, 4.30732, 1.1053},  
	{0.8125, 5.55914, 0.271029, 4.2188, 1.06931},  
	{0.8175, 5.42309, 0.258903, 4.13011, 1.03407},  
	{0.8225, 5.28814, 0.247235, 4.04132, 0.999579},  
	{0.8275, 5.15434, 0.236012, 3.95249, 0.965833},  
	{0.8325, 5.02172, 0.225218, 3.86367, 0.932827},  
	{0.8375, 4.89032, 0.214839, 3.77493, 0.900556},  
	{0.8425, 4.76019, 0.204862, 3.68632, 0.869014},  
	{0.8475, 4.63137, 0.195273, 3.5979, 0.838196},  
	{0.8525, 4.50388, 0.18606, 3.50973, 0.808096},  
	{0.8575, 4.37778, 0.177209, 3.42186, 0.778707},  
	{0.8625, 4.25309, 0.168709, 3.33435, 0.750025},  
	{0.8675, 4.12985, 0.160549, 3.24726, 0.722042},  
	{0.8725, 4.0081, 0.152716, 3.16063, 0.694751},  
	{0.8775, 3.88787, 0.1452, 3.07452, 0.668146},  
	{0.8825, 3.7692, 0.137989, 2.98899, 0.642221},  
	{0.8875, 3.65211, 0.131074, 2.90407, 0.616966},  
	{0.8925, 3.53664, 0.124444, 2.81982, 0.592377},  
	{0.8975, 3.42282, 0.11809, 2.73628, 0.568444},  
	{0.9025, 3.31068, 0.112002, 2.65351, 0.54516},  
	{0.9075, 3.20024, 0.106171, 2.57155, 0.522517},  
	{0.9125, 3.09154, 0.100588, 2.49044, 0.500508},  
	{0.9175, 2.98459, 0.095244, 2.41022, 0.479124},  
	{0.9225, 2.87943, 0.090131, 2.33094, 0.458356},  
	{0.9275, 2.77608, 0.0852408, 2.25264, 0.438197},  
	{0.9325, 2.67456, 0.0805654, 2.17535, 0.418638},  
	{0.9375, 2.57488, 0.0760972, 2.09911, 0.399671},  
	{0.9425, 2.47708, 0.0718287, 2.02397, 0.381286},  
	{0.9475, 2.38117, 0.0677529, 1.94994, 0.363475},  
	{0.9525, 2.28716, 0.0638626, 1.87707, 0.346229},  
	{0.9575, 2.19507, 0.0601511, 1.80538, 0.329538},  
	{0.9625, 2.10492, 0.0566118, 1.73491, 0.313395},  
	{0.9675, 2.01672, 0.0532384, 1.66569, 0.29779},  
	{0.9725, 1.93047, 0.0500248, 1.59774, 0.282713},  
	{0.9775, 1.8462, 0.0469649, 1.53108, 0.268155},  
	{0.9825, 1.76391, 0.0440529, 1.46575, 0.254107},  
	{0.9875, 1.68361, 0.0412832, 1.40176, 0.240561},  
	{0.9925, 1.6053, 0.0386505, 1.33914, 0.227505},  
	{0.9975, 1.52899, 0.0361493, 1.27791, 0.214931},  
	{1.0025, 1.45468, 0.0337747, 1.21808, 0.20283},  
	{1.0075, 1.38238, 0.0315217, 1.15966, 0.191192},  
	{1.0125, 1.31208, 0.0293854, 1.10269, 0.180008},  
	{1.0175, 1.24378, 0.0273613, 1.04716, 0.169268},  
	{1.0225, 1.17749, 0.0254448, 0.993087, 0.158962},  
	{1.0275, 1.1132, 0.0236317, 0.94049, 0.149082},  
	{1.0325, 1.05091, 0.0219176, 0.889373, 0.139617},  
	{1.0375, 0.990601, 0.0202986, 0.839743, 0.130559},  
	{1.0425, 0.932277, 0.0187707, 0.791608, 0.121898},  
	{1.0475, 0.875926, 0.0173301, 0.744972, 0.113624},  
	{1.0525, 0.82154, 0.0159731, 0.699838, 0.105729},  
	{1.0575, 0.769106, 0.0146961, 0.656208, 0.0982023},  
	{1.0625, 0.718614, 0.0134957, 0.614083, 0.0910356},  
	{1.0675, 0.670049, 0.0123686, 0.573461, 0.0842193},  
	{1.0725, 0.623397, 0.0113115, 0.534342, 0.0777443},  
	{1.0775, 0.578643, 0.0103213, 0.49672, 0.0716016},  
	{1.0825, 0.535769, 0.00939502, 0.460592, 0.065782},  
	{1.0875, 0.494757, 0.00852978, 0.425951, 0.0602766},  
	{1.0925, 0.455589, 0.00772275, 0.39279, 0.0550764},  
	{1.0975, 0.418244, 0.00697122, 0.3611, 0.0501727},  
	{1.1025, 0.3827, 0.00627258, 0.330871, 0.0455567},  
	{1.1075, 0.348937, 0.00562432, 0.302093, 0.0412197},  
	{1.1125, 0.31693, 0.00502399, 0.274753, 0.0371532},  
	{1.1175, 0.286656, 0.00446926, 0.248839, 0.0333485},  
	{1.1225, 0.25809, 0.00395786, 0.224335, 0.0297974},  
	{1.1275, 0.231205, 0.00348762, 0.201226, 0.0264915},  
	{1.1325, 0.205975, 0.00305643, 0.179496, 0.0234226},  
	{1.1375, 0.182371, 0.00266228, 0.159127, 0.0205825},  
	{1.1425, 0.160367, 0.00230321, 0.140101, 0.0179633},  
	{1.1475, 0.139933, 0.00197735, 0.122398, 0.0155571},  
	{1.1525, 0.121038, 0.0016829, 0.105999, 0.0133561},  
	{1.1575, 0.103652, 0.00141812, 0.0908813, 0.0113526},  
	{1.1625, 0.0877443, 0.00118135, 0.077024, 0.009539},  
	{1.1675, 0.0732828, 0.000970983, 0.0644039, 0.00790791},  
	{1.1725, 0.060235, 0.000785476, 0.0529975, 0.006452},  
	{1.1775, 0.0485679, 0.000623352, 0.0427805, 0.00516404},  
	{1.1825, 0.0382482, 0.000483191, 0.033728, 0.00403696},  
	{1.1875, 0.0292418, 0.000363631, 0.0258143, 0.0030638},  
	{1.1925, 0.0215143, 0.000263365, 0.0190132, 0.00223773},  
	{1.1975, 0.0150311, 0.000181142, 0.0132979, 0.00155203},  
	{1.2025, 0.00975689, 0.000115761, 0.00864098, 0.00100014},  
	{1.2075, 0.00565625, 6.60733e-05, 0.00501457, 0.000575612},  
	{1.2125, 0.00269342, 3.09793e-05, 0.00239032, 0.000272122},  
	{1.2175, 0.000832356, 9.42694e-06, 0.000739439, 8.34904e-05},  
	{1.2225, 3.68293e-05, 4.10745e-07, 3.27508e-05, 3.66773e-06},  
	{1.2275, 0.000270412, 2.96993e-06, 0.000240705, 2.67372e-05},  
	{1.2325, 0.00149653, 1.61872e-05, 0.00133343, 0.000146917},  
	{1.2375, 0.00367851, 3.91872e-05, 0.00328076, 0.000358558},  
	{1.2425, 0.00677957, 7.11355e-05, 0.00605229, 0.00065615},  
	{1.2475, 0.0107629, 0.000111237, 0.00961738, 0.00103431},  
	{1.2525, 0.0155918, 0.000158734, 0.0139452, 0.00148781},  
	{1.2575, 0.0212293, 0.000212907, 0.0190049, 0.00201153},  
	{1.2625, 0.0276388, 0.000273071, 0.0247652, 0.00260051},  
	{1.2675, 0.0347837, 0.000338576, 0.0311952, 0.00324991},  
	{1.2725, 0.0426274, 0.000408805, 0.0382636, 0.00395504},  
	{1.2775, 0.0511336, 0.000483173, 0.0459391, 0.00471132},  
	{1.2825, 0.0602662, 0.000561126, 0.0541908, 0.00551433},  
	{1.2875, 0.0699892, 0.000642141, 0.0629873, 0.00635978},  
	{1.2925, 0.080267, 0.000725722, 0.0722978, 0.00724349},  
	{1.2975, 0.0910642, 0.000811403, 0.0820913, 0.00816144},  
	{1.3025, 0.102346, 0.000898744, 0.0923371, 0.00910974},  
	{1.3075, 0.114077, 0.000987329, 0.103005, 0.0100846},  
	{1.3125, 0.126223, 0.00107677, 0.114064, 0.0110824},  
	{1.3175, 0.13875, 0.0011667, 0.125484, 0.0120997},  
	{1.3225, 0.151625, 0.00125678, 0.137235, 0.013133},  
	{1.3275, 0.164814, 0.00134669, 0.149289, 0.014179},  
	{1.3325, 0.178286, 0.00143612, 0.161615, 0.0152348},  
	{1.3375, 0.192007, 0.00152481, 0.174185, 0.0162972},  
	{1.3425, 0.205946, 0.00161249, 0.18697, 0.0173634},  
	{1.3475, 0.220073, 0.00169892, 0.199943, 0.0184306},  
	{1.3525, 0.234355, 0.00178389, 0.213075, 0.0194962},  
	{1.3575, 0.248765, 0.00186719, 0.22634, 0.0205577},  
	{1.3625, 0.263272, 0.00194862, 0.23971, 0.0216126},  
	{1.3675, 0.277847, 0.00202803, 0.25316, 0.0226587},  
	{1.3725, 0.292462, 0.00210525, 0.266663, 0.0236937},  
	{1.3775, 0.307091, 0.00218014, 0.280195, 0.0247157},  
	{1.3825, 0.321706, 0.00225259, 0.293731, 0.0257226},  
	{1.3875, 0.336282, 0.00232246, 0.307246, 0.0267127},  
	{1.3925, 0.350792, 0.00238967, 0.320718, 0.0276841},  
	{1.3975, 0.365212, 0.00245411, 0.334123, 0.0286352},  
};











