

#include <psim.h>
#include "../GENERATORS/eta.h"

#include <tcl.h>
#include <tk.h>

Tcl_Interp* interp;
int running=0,stop_running=1;
int REDRAW_BETWEEN_EVENTS;
char *GENERATOR_NAME,*SUBTYPE_NAME,*DECAYMECH_NAME;

int gen_photon,gen_electron,gen_pair,gen_pi0,gen_eta_He4;
int gen_eta_C12,gen_eta_prime_C12,gen_eta_prime_He4,gen_rho,gen_omega;


#include "tcl_procs.h"

Tcl_CmdProc Redraw;
Tcl_CmdProc OneEvent;
Tcl_CmdProc Run;
Tcl_CmdProc Stop;
Tcl_CmdProc IdleThread;
Tcl_CmdProc RedefineGeometry;
Tcl_CmdProc ResetView;
Tcl_CmdProc ResetGenerators;
Tcl_CmdProc ResetHistograms;

char *Subtype_string(int type,int subtype);
char *Decaymech_string(int type,int decay_mech);

void GUI_init(void)
{
   char str[256];

   printf("Launching GUI...\n");
   
	gen_photon        =        photon_weight==0.0 ? 0:1;
	gen_electron      =      electron_weight==0.0 ? 0:1;
	gen_pair          =          pair_weight==0.0 ? 0:1;
	gen_pi0           =           pi0_weight==0.0 ? 0:1;
	gen_eta_He4       =       eta_He4_weight==0.0 ? 0:1;
	gen_eta_C12       =       eta_C12_weight==0.0 ? 0:1;
	gen_eta_prime_C12 = eta_prime_C12_weight==0.0 ? 0:1;
	gen_eta_prime_He4 = eta_prime_He4_weight==0.0 ? 0:1;
	gen_rho           =           rho_weight==0.0 ? 0:1;
	gen_omega         =         omega_weight==0.0 ? 0:1;

   /* Tcl/Tk */
   interp=Tcl_CreateInterp();
   if(!interp){printf("Unable to create Tcl/Tk interpreter!\n");exit(-1);}
   Tcl_Init(interp);
   Tk_Init(interp);

   /* Create Tcl commands */
   Tcl_CreateCommand(interp,"Redraw",Redraw,0,NULL);
   Tcl_CreateCommand(interp,"RedefineGeometry",RedefineGeometry,0,NULL);
   Tcl_CreateCommand(interp,"OneEvent",OneEvent,0,NULL);
   Tcl_CreateCommand(interp,"Run",Run,0,NULL);
   Tcl_CreateCommand(interp,"Stop",Stop,0,NULL);
   Tcl_CreateCommand(interp,"IdleThread",IdleThread,0,NULL);
   Tcl_CreateCommand(interp,"ResetView",ResetView,0,NULL);
   Tcl_CreateCommand(interp,"ResetGenerators",ResetGenerators,0,NULL);
   Tcl_CreateCommand(interp,"ResetHistograms",ResetHistograms,0,NULL);

   /* Setup some Tcl globals */
   flags[draw_tracks]=1;
   sprintf(str,"%f",vtheta);Tcl_SetVar(interp,"vtheta",str,TCL_GLOBAL_ONLY);
   sprintf(str,"%f",vphi);Tcl_SetVar(interp,"vphi",str,TCL_GLOBAL_ONLY);
   sprintf(str,"%f",vpsi);Tcl_SetVar(interp,"vpsi",str,TCL_GLOBAL_ONLY);
   sprintf(str,"%f",vu);Tcl_SetVar(interp,"vu",str,TCL_GLOBAL_ONLY);
   sprintf(str,"%f",vv);Tcl_SetVar(interp,"vv",str,TCL_GLOBAL_ONLY);
   sprintf(str,"%f",vsu);Tcl_SetVar(interp,"vsu",str,TCL_GLOBAL_ONLY);
   sprintf(str,"%f",vsv);Tcl_SetVar(interp,"vsv",str,TCL_GLOBAL_ONLY);

   Tcl_LinkVar(interp,"ievent",(char*)&IEVENT,TCL_LINK_INT);
   Tcl_LinkVar(interp,"hide",(char*)&HIDE,TCL_LINK_INT);
   Tcl_LinkVar(interp,"shade",(char*)&SHADE,TCL_LINK_INT);
   Tcl_LinkVar(interp,"nct",(char*)&flags[dont_draw_charged_tracks],TCL_LINK_INT);
   Tcl_LinkVar(interp,"nnt",(char*)&flags[dont_draw_neutral_tracks],TCL_LINK_INT);
   Tcl_LinkVar(interp,"rbe",(char*)&REDRAW_BETWEEN_EVENTS,TCL_LINK_INT);
   Tcl_LinkVar(interp,"generate_only",(char*)&flags[generate_only],TCL_LINK_INT);
   Tcl_LinkVar(interp,"stop_at_calorimeter",(char*)&flags[stop_at_calorimeter],TCL_LINK_INT);
   Tcl_LinkVar(interp,"event_type",(char*)&event_type,TCL_LINK_INT);
   Tcl_LinkVar(interp,"event_subtype",(char*)&event_subtype,TCL_LINK_INT);
   Tcl_LinkVar(interp,"event_decay_mech",(char*)&event_decay_mech,TCL_LINK_INT);
   Tcl_LinkVar(interp,"npart_infos",(char*)&Npart_infos,TCL_LINK_INT);

   Tcl_LinkVar(interp,"gen_photon",(char*)&gen_photon,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_electron",(char*)&gen_electron,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_pair",(char*)&gen_pair,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_pi0",(char*)&gen_pi0,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_eta_he4",(char*)&gen_eta_He4,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_eta_c12",(char*)&gen_eta_C12,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_eta_prime_c12",(char*)&gen_eta_prime_C12,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_eta_prime_he4",(char*)&gen_eta_prime_He4,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_rho",(char*)&gen_rho,TCL_LINK_INT);
   Tcl_LinkVar(interp,"gen_omega",(char*)&gen_omega,TCL_LINK_INT);


   GENERATOR_NAME=Tcl_Alloc(256); strcpy(GENERATOR_NAME,"None");
   Tcl_LinkVar(interp,"generator_name",(char*)&GENERATOR_NAME,TCL_LINK_STRING);
   SUBTYPE_NAME=Tcl_Alloc(256); strcpy(SUBTYPE_NAME,"None");
   Tcl_LinkVar(interp,"subtype_name",(char*)&SUBTYPE_NAME,TCL_LINK_STRING);
   DECAYMECH_NAME=Tcl_Alloc(256); strcpy(DECAYMECH_NAME,"None");
   Tcl_LinkVar(interp,"decaymech_name",(char*)&DECAYMECH_NAME,TCL_LINK_STRING);

   /* Read Tcl initialization script */
   Tcl_GlobalEval(interp,psim_gui_tcl_proc);
   /*Tcl_EvalFile(interp,"psim_gui.tcl");*/
   printf("%s\n",interp->result);

   /*----------------- main loop -----------------*/
   Tk_MainLoop();

   stop_running=1;
}



int Redraw(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   
   vtheta=atof(Tcl_GetVar(interp,"vtheta",TCL_GLOBAL_ONLY));
   vphi=atof(Tcl_GetVar(interp,"vphi",TCL_GLOBAL_ONLY));
   vpsi=atof(Tcl_GetVar(interp,"vpsi",TCL_GLOBAL_ONLY));
   vu=atof(Tcl_GetVar(interp,"vu",TCL_GLOBAL_ONLY));
   vv=atof(Tcl_GetVar(interp,"vv",TCL_GLOBAL_ONLY));
   vsu=atof(Tcl_GetVar(interp,"vsu",TCL_GLOBAL_ONLY));
   vsv=atof(Tcl_GetVar(interp,"vsv",TCL_GLOBAL_ONLY));
   
   draw_all();
   
   return TCL_OK;
}

int OneEvent(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   flags[quit]=1;

   if(running)return TCL_OK;
   running=1;

   if(REDRAW_BETWEEN_EVENTS)Tcl_Eval(interp,".top.left.redraw invoke");
   grun_();
   
   running=0;
   return TCL_OK;
}

int Run(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   stop_running=0;
   while(!stop_running){
      Tcl_Eval(interp,"update");
      OneEvent(clientData,interp,argc,argv);
   }
   return TCL_OK;
}

int Stop(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   stop_running=1;
   return TCL_OK;
}

int IdleThread(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   Tcl_UpdateLinkedVar(interp,"ievent");
   Tcl_UpdateLinkedVar(interp,"event_type");
   Tcl_UpdateLinkedVar(interp,"event_subtype");
   Tcl_UpdateLinkedVar(interp,"event_decay_mech");
   strcpy(GENERATOR_NAME,generator_names[event_type]);
   Tcl_UpdateLinkedVar(interp,"generator_name");
   strcpy(SUBTYPE_NAME,Subtype_string(event_type,event_subtype));
   Tcl_UpdateLinkedVar(interp,"subtype_name");
   strcpy(DECAYMECH_NAME,Decaymech_string(event_type,event_decay_mech));
   Tcl_UpdateLinkedVar(interp,"decaymech_name");
   Tcl_UpdateLinkedVar(interp,"npart_infos");

   Tcl_Eval(interp,"after 200 IdleThread");

   return TCL_OK;
}

int RedefineGeometry(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   

   return TCL_OK;
}

char *Subtype_string(int type,int subtype)
{
	static char str[32];
	
	switch(type){
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
			if(subtype==TOTAL_CROSS)return "Total cross";
			if(subtype==PRIM_CROSS )return "Primakoff";
			if(subtype==NUCL_CROSS )return "Nuclear Coh.";
			if(subtype==INTER_CROSS)return "Interference";
			break;
		case 8:
		case 9:
			return "phasespace";
			break;
		default:
			sprintf(str,"%d",subtype);
			return str;
	}
	return "Unknown";
}

char *Decaymech_string(int type,int decay_mech)
{
	static char str[32];
	
	switch(type){
		case 3:
			if(decay_mech==0)return "pi0 -> 2gamma";
		case 4:
		case 5:
			if(decay_mech==0 )return "eta -> 2gamma";
			if(decay_mech==1 )return "eta -> 3pi0";
			if(decay_mech==2 )return "eta -> pi+ pi- pi0";
			if(decay_mech==3 )return "eta -> pi+ pi- gamma";
			break;
		case 6:
		case 7:
			if(decay_mech==0 )return "eta' -> pi+ pi- eta";
			if(decay_mech==1 )return "eta' -> rho gamma";
			if(decay_mech==2 )return "eta' -> 2pi0 eta";
			if(decay_mech==3 )return "eta' -> omega gamma";
			if(decay_mech==4 )return "eta' -> gamma gamma";
			break;
		case 8:
			return "rho -> pi+ pi-";
			break;
		case 9:
			if(decay_mech==0 )return "omega -> pi+ pi- pi0";
			if(decay_mech==1 )return "omega -> pi0 gamma";
			if(decay_mech==2 )return "omega -> pi+ pi-";
			break;
		default:
			sprintf(str,"%d",decay_mech);
			return str;
	}
	return "Unknown";
}


int ResetView(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
   char cmd[256];
	int view;

	view=atoi(argv[1]);

	switch(view){
		case 1:
			vtheta=45.0;
			vphi  =45.0;
			vpsi  = 0.0;
			vu    =18.0;
			vv    =18.0;
			vsu   =0.025;
			vsv   =0.025;
			break;
		case 2:
			vtheta=90.0;
			vphi  =0.0;
			vpsi  =0.0;
			vu    =50.0;
			vv    =10.0;
			vsu   =0.2;
			vsv   =0.2;
			break;
		case 3:
			vtheta=90.0;
			vphi  =0.0;
			vpsi  =0.0;
			vu    =22.0;
			vv    =10.0;
			vsu   =0.02;
			vsv   =0.1;
			break;
		case 4:
			vtheta=179.9;
			vphi  =45.0;
			vpsi  =0.0;
			vu    =10.0;
			vv    =10.0;
			vsu   =0.1;
			vsv   =0.1;
			break;
		case 5:
			vtheta=90.0;
			vphi  =90.0;
			vpsi  = 0.0;
			vu    =-2.0;
			vv    =10.0;
			vsu   =0.02;
			vsv   =0.1;
			break;
		case 6:
			vtheta= 0.0;
			vphi  =90.0;
			vpsi  = 0.0;
			vu    =10.0;
			vv    =10.0;
			vsu   =0.1;
			vsv   =0.1;
			break;
	}

   sprintf(cmd,"set vtheta %f",vtheta); Tcl_Eval(interp,cmd);
   sprintf(cmd,"set vphi %f",vphi); Tcl_Eval(interp,cmd);
   sprintf(cmd,"set vpsi %f",vpsi); Tcl_Eval(interp,cmd);
   sprintf(cmd,"set vu %f",vu); Tcl_Eval(interp,cmd);
   sprintf(cmd,"set vv %f",vv); Tcl_Eval(interp,cmd);
   sprintf(cmd,"set vsu %f",vsu); Tcl_Eval(interp,cmd);
   sprintf(cmd,"set vsv %f",vsv); Tcl_Eval(interp,cmd);

	Tcl_Eval(interp,".top.left.redraw invoke"); 
   
   return TCL_OK;
}


int ResetGenerators(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
	float total;
	
	       photon_weight=gen_photon        ? 1.0 : 0.0;
	     electron_weight=gen_electron      ? 1.0 : 0.0;
	         pair_weight=gen_pair          ? 1.0 : 0.0;
	          pi0_weight=gen_pi0           ? 1.0 : 0.0;
	      eta_He4_weight=gen_eta_He4       ? 1.0 : 0.0;
	      eta_C12_weight=gen_eta_C12       ? 1.0 : 0.0;
	eta_prime_C12_weight=gen_eta_prime_C12 ? 1.0 : 0.0;
	eta_prime_He4_weight=gen_eta_prime_He4 ? 1.0 : 0.0;
	          rho_weight=gen_rho           ? 1.0 : 0.0;
	        omega_weight=gen_omega         ? 1.0 : 0.0;

	total =   photon_weight
				+electron_weight
				+pair_weight
				+pi0_weight
				+eta_He4_weight
				+eta_C12_weight
				+eta_prime_C12_weight
				+eta_prime_He4_weight
				+rho_weight
				+omega_weight;
	if(total<=0.0){
		photon_weight=1.0;
		gen_photon=1;
		Tcl_UpdateLinkedVar(interp,"gen_photon");
	}

	init_generators();

	return TCL_OK;
}

int ResetHistograms(ClientData clientData,Tcl_Interp *interp,int argc,char *argv[])
{
	hreset(0," ");
	return TCL_OK;
}




