

#include "psim.h"

void gsatt(char *volu,char *attr,int ival)
{gsatt_(volu,attr,&ival,strlen(volu),strlen(attr));}

void uginit(void)
{
   float X0=0.,Y0=0.,Z0=0.,AXSIZ=50.;
   float U0=18.0,V0=18.0;
   FILE *file;
   int file_exists=1;
   
   /*float theta=45.0, phi=45.0, psi=0.0;
   float SU=0.032, SV=0.032;
   */
   /* seed random number generator */
   srandom(time(NULL));
      
   /* initialize graphics */
   if(flags[graphics])hplint(1); /* 7879=xterm 1-10= X-window */

   /* initialize GEANT */
   if(flags[verbose])printf("\n\t\t--------Initializing GEANT--------\n\n");
   ginit_();
   
   /* set default values */
   defaults1_(&GEOMETRY_TYPE);

   /* read data records */
   if(flags[verbose])printf("\n\t\t------Reading Data Records--------\n\n");
   if(NULL==(file=fopen("pr_back.dat","r"))){
      printf("\nNo \"pr_back.dat\" file found!\n\n");
      file_exists=0;
   }else{
   	fclose(file);
   	datarecords_();
   }
   
   /* set default values */
   defaults2_();

   /* initialize data structure and graphics */
   if(flags[verbose])printf("\n\t\t---Initializing data structures---\n\n");
   gzinit_();
   if(flags[graphics])gdinit_();
   
   /* particle table initialization */
   if(flags[verbose])printf("\n\t\t---Initializing particle table----\n\n");
   gpart_();
   
   /* Geometry and materials description */
   if(flags[verbose])printf("\n\t\t--------Defining Geometry--------\n\n");
   ugeom();
   ggclos_();
   
   /* Energy loss and cross-section tables */
   if(flags[verbose])printf("\n\t\t-Initializing energy loss and X-sct tables-\n\n");
   gphysi_();
   
   /* print table information */
   if(flags[verbose]){
      gprint_("MATE",&zero,strlen("MATE"));
      gprint_("TMED",&zero,strlen("TMED"));
      gprint_("VOLU",&zero,strlen("VOLU"));
   }
   
   /* draw detector setup */
   if(flags[graphics]){
      draw_all();
   }
}

/* Draw all detectors using specifications in global variables */
void draw_all(void)
{
   float X0=0.,Y0=0.,Z0=0.,AXSIZ=50.;
   float U0=18.0,V0=18.0;

   ixclrwi_();

      printf("\n\tDrawing detector...\n");
      gsatt("*","SEEN",0);
      gsatt("TTSW","SEEN",1);
      gsatt("TTSW","FILL",7);
      gsatt("TTSW","COLO",2);

      if(GEOMETRY_TYPE==0){
         gsatt("TTIN","SEEN",1);
         gsatt("TTIN","FILL",7);
         gsatt("TTIN","COLO",4);

         gsatt("PTRG","SEEN",1);
      }else if(GEOMETRY_TYPE==1 || GEOMETRY_TYPE==2){
         gsatt("TFRM","SEEN",1);
         gsatt("TFRM","FILL",7);
         gsatt("TFRM","COLO",6);
         gsatt("TGAS","SEEN",1);      
         gsatt("TGAS","FILL",7);
         gsatt("TGAS","COLO",1);
      }
      
      gsatt("DPOL","SEEN",1);
      gsatt("DPOL","FILL",7);
      gsatt("DPOL","COLO",3);

      gsatt("DVGP","SEEN",1);
      gsatt("DVGP","FILL",7);
      gsatt("DVGP","COLO",0);
      
      gsatt("DVSL","SEEN",1);
      gsatt("DVSL","FILL",7);
      gsatt("DVSL","COLO",7);
      
      gsatt("PCUP","SEEN",1);
      gsatt("PCUP","FILL",7);
      gsatt("PCUP","COLO",2);
      
      gsatt("PCDN","SEEN",1);
      gsatt("PCDN","FILL",7);
      gsatt("PCDN","COLO",2);
      
      gsatt("VABW","SEEN",0);
      gsatt("VABW","FILL",7);
      gsatt("VABW","COLO",6);

      gsatt("VPIP","SEEN",1);
      gsatt("VPIP","FILL",7);
      gsatt("VPIP","COLO",6);

      gdopt_("HIDE",HIDE ? "ON  ":"OFF ",strlen("HIDE"),strlen("ON  "));
      gdopt_("SHAD",SHADE ? "ON  ":"OFF ",strlen("SHAD"),strlen("ON  "));
      
      gdraw_("HALB",&vtheta,&vphi,&vpsi,&vu,&vv,&vsu,&vsv,strlen("HALB"));
      
      U0=1.0;V0=5.0;
      gdscal_(&U0,&V0);
      gdaxis_(&X0,&Y0,&Z0,&AXSIZ);

      ixupdwi_(&zero); /* update window */

}








