


#include "psim.h"


/* set detector trheshold at 1 MeV */
#define E_thresh 1.0E-3

subset *sets[max_sets];
int nsets=0;

int announce_subsets=1;

/****************************************************************/
/* Create a new "subset" of detectors. The total energy and     */
/* earliest time detected by all the members of the subset      */
/* will be determined at the end of each event.                 */
/****************************************************************/
subset* AddSubset(char *name){
   subset *s;
   char chform[512],chblok[32],lcname[32],hits_str[32];
   int i;
   
   if(announce_subsets)printf("Creating subset \"%s\"\n",name);
   /* allocate memory for subset data structure */
   s=(subset*)malloc(sizeof(subset));
   if(s==NULL){
      printf("error allocating memory for subset!\n");
      exit(-1);
      return NULL;
   }
   
   /* add subset to list of sets */
   sets[nsets++]=s;
   s->Num=0;
   strcpy(s->name,name);

/*--- This put all subsets into a CWN ntuple ---*/
#if 0
   /* prepare some strings */
   strcpy(chblok,name);
   
   /* use lower case of set name as prefix for variable names */
   strcpy(lcname,name);
   for(i=0;i<strlen(lcname);i++)lcname[i]=(char)tolower((int)lcname[i]);
   
   /* name of variable representing number of hits/elements in arrays */
   sprintf(hits_str,"%s_hits",lcname);
   
   /* string defining variables to be added to ntuple */
   sprintf(chform,"%s[0,1023]:I,%s_E:R,%s_T:R,%s_Eelem(%s):R,%s_Telem(%s):R"
      ,hits_str,lcname,lcname,lcname,hits_str,lcname,hits_str);
   
   /* add the subsets for this variable to the ntuple */
   hbname(nt1,chblok,&(s->HITS),chform);
#endif
   
   /* return a pointer to the newly allocated subset structure */
   /* so the "Num" and "list" fields can be filled */
   return s;

}

/********************************************************/
/* Return the pointer to the subset structure given the */
/* subset name. Exit with an error message if the       */
/* subset doesn't exist.                                */
/********************************************************/
subset* FindSubset(char *name)
{
   int i;
   subset *s;
   
   for(i=0;i<nsets;i++){
      s=sets[i];
      if(!strcmp(s->name,name))return s;
   }
   
   printf("\n\nUnknown subset \"%s\" passed to FindSubset().\n\n",name);
   exit(-1);
   
   return NULL;
}

/********************************************/
/* Return whether a subset currently exists */
/********************************************/
int SubsetExists(char *name)
{
   int i;
   subset *s;
   
   for(i=0;i<nsets;i++){
      s=sets[i];
      if(!strcmp(s->name,name))return 1;
   }

   return 0;
}

/*************************************************************/
/* Add a block of detectors to the subset pointed to by s.   */
/* The block will extend dx detectors in x and dy detectors  */
/* in y and will have the upper left corner at position x,y. */
/* The values x and y are in detector units (i.e. integers   */
/* from 1 to 27).                                            */
/* If "inner" is non-zero, then the block is taken to be     */
/* made of the insertion detectors, otherwise, it is taken   */
/* to be made of the outer lead-glass detectors.             */
/*************************************************************/
void AddBlockToSubset(subset *s,int x,int y,int dx,int dy,int inner)
{
   int i,j;
   
   if(!s){
      printf("\n\n Bad subset pointer passed to AddBlockToSubset().\n\n");
      exit(-1);
   }
   
   for(i=x;i<x+dx;i++){
      for(j=y;j<y+dy;j++){
         if(!inner){
            if(i<1 || i>27 || j<1 || j>27)continue;
            s->list[s->Num++]=((j-1)*27)+i;
         }else{
            if(i<1 || i>22 || j<1 || j>22)continue;
            s->list[s->Num++]=((j-1)*22)+i+1000;
         }
      }
   }
}

/**************************************************************/
/* Print a list of subset names which contain detector "who". */
/* This is just intended for diagnostics.                     */
/**************************************************************/
void ListSubsets(int who)
{
   int i,j;
   subset *s;
   int is_member_of_a_subset=0;
   
   printf("Subsets for %d:",who);
   for(i=0;i<nsets;i++){
      s=sets[i];
      for(j=0;j<s->Num;j++){
         if(s->list[j]==who){
            printf("\t%s",s->name);
            is_member_of_a_subset=1;
         }
      }
   }
   if(!is_member_of_a_subset)printf(" none");
   printf("\n");
}


/*****************************************************/
/* Complete the subset data structures for the event */
/* using SHWDET and TSHWDET. Return a 1 if at least  */
/* one detector was above E_thresh (defined at top   */
/* of page), zero otherwise.                         */
/*****************************************************/
int ProcessSubsets(float *SHWDET,float *TSHWDET)
{
   int i,j;
   subset *s;
   int something_hit=0;

   /* fill subsets data structure */
   for(j=0;j<nsets;j++){
      s=sets[j];
      s->HITS=0;
      s->T=1.0E10;
      s->E=0.0;
      for(i=0;i<s->Num;i++){
         /* sparsely fill arrays only with detectors that have energy */
         if(SHWDET[s->list[i]] > E_thresh){
            s->Eelem[s->HITS] = SHWDET[s->list[i]]; /* GeV */
            s->Telem[s->HITS] = TSHWDET[s->list[i]];
         
            /* add to total energy of set */
            s->E+=s->Eelem[s->HITS];
            
            /* use time of first detector hit for time of set */
            if(s->Telem[s->HITS]<s->T)s->T=s->Telem[s->HITS];
            
            s->HITS++;
            something_hit=1;
         }
      }
   }
   
   return something_hit;
}








