

#include "psim.h"



int MLU_T2[65536];


void DefineGroups2(void);
void ProgramMLUs2(void);



/**********************************************/
/* Initialize stuff for the trigger simulator */
/**********************************************/
void init_trigger2(void)
{
   /* Linear Fan-in Groups */
   DefineGroups2();
   
   /* Program MLUs */
   ProgramMLUs2();


}


/************************************************************/
/* Define Group subsets. A "group" here represents a group  */
/* of detectors added together in a UVa120 linear fan-in.   */
/* Some groups will be added together in UVa125 adder/      */
/* discriminator modules. This is done in trigger() below.  */
/************************************************************/
void DefineGroups2(void)
{
   subset *s;
   int i,M,k,*x,*y,*dx,*dy;
   char str[256];
   int first,last;
   int *x_ptrs[128],*y_ptrs[128];
   int *dx_ptrs[128],*dy_ptrs[128];
   int N=0;


   /* outer */
   int  x1[]={  1, 15,  1, 14,  1,  15,  1, 14, 0};
   int  y1[]={  1,  1,  3,  3,  5,   5,  7,  7, 0};
   int dx1[]={ 14, 13, 13, 14, 14,  13, 13, 14, 0};
   int dy1[]={  2,  2,  2,  2,  2,   2,  2,  2, 0};

   int  x2[]={  1,  1,  1,  1,  1,  1, 0};
   int  y2[]={  9, 11, 13, 15, 16, 18, 0};
   int dx2[]={ 27, 27, 27, 27, 27, 27, 0};
   int dy2[]={  2,  2,  2,  1,  2,  2, 0};

   int  x3[]={  1, 15,  1, 14,  1,  15,  1, 14, 0};
   int  y3[]={ 20, 20, 22, 22, 24,  24, 26, 26, 0};
   int dx3[]={ 14, 13, 13, 14, 14,  13, 13, 14, 0};
   int dy3[]={  2,  2,  2,  2,  2,   2,  2,  2, 0};
   

   /* inner */
   int  x4[]={  1,  8, 16,  1,  8, 16,  1,  8, 16, 0};
   int  y4[]={  1,  1,  1,  5,  5,  5,  9,  9,  9, 0};
   int dx4[]={  7,  8,  7,  7,  8,  7,  7,  8,  7, 0};
   int dy4[]={  4,  4,  4,  4,  4,  4,  4,  4,  4, 0};
   
   int  x5[]={  1, 12,  1,  8, 16,  1,  8, 16, 0};
   int  y5[]={ 13, 13, 15, 15, 15, 19, 19, 19, 0};
   int dx5[]={ 11, 11,  7,  8,  7,  7,  8,  7, 0};
   int dy5[]={  2,  2,  4,  4,  4,  4,  4,  4, 0};
   
   printf("Defining Groups for trigger 2\n");
   x_ptrs[N]=x1; y_ptrs[N]=y1; dx_ptrs[N]=dx1; dy_ptrs[N++]=dy1;
   x_ptrs[N]=x2; y_ptrs[N]=y2; dx_ptrs[N]=dx2; dy_ptrs[N++]=dy2;
   x_ptrs[N]=x3; y_ptrs[N]=y3; dx_ptrs[N]=dx3; dy_ptrs[N++]=dy3;
   x_ptrs[N]=x4; y_ptrs[N]=y4; dx_ptrs[N]=dx4; dy_ptrs[N++]=dy4;
   x_ptrs[N]=x5; y_ptrs[N]=y5; dx_ptrs[N]=dx5; dy_ptrs[N++]=dy5;

   M=0;
   for(i=0;i<3;i++){
      k=-1; x=x_ptrs[i]; y=y_ptrs[i]; dx=dx_ptrs[i]; dy=dy_ptrs[i];
      while(x[++k]){
         sprintf(str,"T2V%d",++M);
         AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],0);
         sprintf(str,"T2H%d",M);
         AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],0);
      }
   }

   for(  ;i<N;i++){
      k=-1; x=x_ptrs[i]; y=y_ptrs[i]; dx=dx_ptrs[i]; dy=dy_ptrs[i];
      while(x[++k]){
         sprintf(str,"T2V%d",++M);
         AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],1);
         sprintf(str,"T2H%d",M);
         AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],1);
      }
   }


   if(debug>1){
      /* show subset lists */
      for(i=1;i<=729;i++){
         ListSubsets(i);
      }
      for(i=1001;i<=1484;i++){
         ListSubsets(i);
      }
   }
}


/****************************************************************/
/* Fill the arrays with the same values that would be programed */
/* in the memory look-up unit  used in the trigger electronics  */
/* This trigger scheme will only require one MLU stage with     */
/* the outputs ORed in a conventional NIM module.               */
/****************************************************************/
void ProgramMLUs2(void)
{
   int i,j,k;
   int tmp;
   
   /* initalize */
   for(i=0;i<65536;i++)MLU_T2[i]=0;
      
   /* set bit 1 whenever two bits set which are separated */
   /* by at least one bit */
   for(i=0;i<=11;i++){
      for(j=i+2;j<=13;j++){
         tmp=(1<<i) + (1<<j); /* two bits separated by one or more bits */
         for(k=0;k<65536;k++)MLU_T2[tmp|k]|=1<<0;
      }
   }
      
}



int trigger2(float Ethresh,int* XBITS,int* YBITS)
{
   float E,E1,E2,E3;
   int xbits,ybits;
   int nxbits,nybits;
   int i;
   char str[256];
   char g1[32],g2[32];
   float Etrigger_thresh=Ethresh/2.0;
   
   /* Note: 
   /* Ethresh is the minimum energy photon we wish to */
   /*  accept in our trigger.
   /*
   /* Etrigger_thresh is the discriminator threshold  */
   
   
   /*********************************************************/
   /* Emulate the trigger electronics.                      */
   /*********************************************************/
   xbits=ybits=0;
   nxbits=nybits=0;

   /* Outer Strips */
   for(i=0;i<4;i++){
      sprintf(g1,"T2V%d",(2*i)+1);
      sprintf(g2,"T2V%d",(2*i)+2);
      E=FindSubset(g1)->E + FindSubset(g2)->E;
      if(E>Etrigger_thresh){ xbits|=(1<<i); nxbits++; }

      sprintf(g1,"T2H%d",(2*i)+1);
      sprintf(g2,"T2H%d",(2*i)+2);
      E=FindSubset(g1)->E + FindSubset(g2)->E;
      if(E>Etrigger_thresh){ ybits|=(1<<i); nybits++; }
   }

   for(i=7;i<11;i++){
      sprintf(g1,"T2V%d",(2*i)+1);
      sprintf(g2,"T2V%d",(2*i)+2);
      E=FindSubset(g1)->E + FindSubset(g2)->E;
      if(E>Etrigger_thresh){ xbits|=(1<<i+3); nxbits++; }

      sprintf(g1,"T2H%d",(2*i)+1);
      sprintf(g2,"T2H%d",(2*i)+2);
      E=FindSubset(g1)->E + FindSubset(g2)->E;
      if(E>Etrigger_thresh){ ybits|=(1<<i+3); nybits++; }
   }
   
   /* Inner Strips */
   E1=FindSubset("T2V9")->E;
   E2=FindSubset("T2V23")->E + FindSubset("T2V24")->E + FindSubset("T2V25")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ xbits|=(1<<4); nxbits++; }

   E1=FindSubset("T2V10")->E;
   E2=FindSubset("T2V26")->E + FindSubset("T2V27")->E + FindSubset("T2V28")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ xbits|=(1<<5); nxbits++; }

   E1=FindSubset("T2V11")->E;
   E2=FindSubset("T2V29")->E + FindSubset("T2V30")->E + FindSubset("T2V31")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ xbits|=(1<<6); nxbits++; }

   E1=FindSubset("T2V12")->E;
   E2=FindSubset("T2V32")->E + FindSubset("T2V33")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ xbits|=(1<<7); nxbits++; }

   E1=FindSubset("T2V13")->E;
   E2=FindSubset("T2V34")->E + FindSubset("T2V35")->E + FindSubset("T2V36")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ xbits|=(1<<8); nxbits++; }

   E1=FindSubset("T2V14")->E;
   E2=FindSubset("T2V37")->E + FindSubset("T2V38")->E + FindSubset("T2V39")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ xbits|=(1<<9); nxbits++; }


   E1=FindSubset("T2H9")->E;
   E2=FindSubset("T2H23")->E + FindSubset("T2H24")->E + FindSubset("T2H25")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ ybits|=(1<<4); nybits++; }

   E1=FindSubset("T2H10")->E;
   E2=FindSubset("T2H26")->E + FindSubset("T2H27")->E + FindSubset("T2H28")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ ybits|=(1<<5); nybits++; }

   E1=FindSubset("T2H11")->E;
   E2=FindSubset("T2H29")->E + FindSubset("T2H30")->E + FindSubset("T2H31")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ ybits|=(1<<6); nybits++; }

   E1=FindSubset("T2H12")->E;
   E2=FindSubset("T2H32")->E + FindSubset("T2H33")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ ybits|=(1<<7); nybits++; }

   E1=FindSubset("T2H13")->E;
   E2=FindSubset("T2H34")->E + FindSubset("T2H35")->E + FindSubset("T2H36")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ ybits|=(1<<8); nybits++; }

   E1=FindSubset("T2H14")->E;
   E2=FindSubset("T2H37")->E + FindSubset("T2H38")->E + FindSubset("T2H39")->E;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh){ ybits|=(1<<9); nybits++; }
  
   

   /* Display bits if in debug mode */
   if(debug>1){
      printf("\n");
      printf(" T2 xbits(%d): ",nxbits);
      for(i=0;i<8;i++)printf("%d ",(xbits>>i)&1);
      printf("\n T2 ybits(%d): ",nybits);
      for(i=0;i<8;i++)printf("%d ",(ybits>>i)&1);
      printf("\n");
      if(debug>2){
         for(i=1;i<=39;i++){
            sprintf(str,"T2V%d",i); printf("\t%s:%3.2f  ",str,FindSubset(str)->E);
            sprintf(str,"T2H%d",i); printf("\t%s:%3.2f\n",str,FindSubset(str)->E);
         }
      }
   }   
 
   if(XBITS)*XBITS=xbits;
   if(YBITS)*YBITS=ybits;
   
   return MLU_T2[xbits&0x3FFF]>0 || MLU_T2[ybits&0x3FFF]>0 ;
}









