




#include "psim.h"


#define Ethresh 0.0

char fname[256],outfname[256]="";
FILE *gzfile;
int N_events_to_add=0;
int quit_program=0;

int psim_readfile_init(char *fname);
void psim_readfile_close(void);
float psim_readfile_percent_done(void);
int read_event(raw_event_t *re);

void ParseArgs(int narg,char *argv[]);
void AddEvent(raw_event_t *re1,raw_event_t *re2);
void Usage(void);


int main(int narg,char *argv[])
{
   raw_event_t re,re_tot;
   int i,j,k;
   int Nevents,Nevents_written=0;
   long Nevents_generated=0;
   time_t last_time=0L;
   part_info_t *p;
   FILE *outfile;
   int end_of_file;
   long last_event=0;
   
   ParseArgs(narg,argv);
   
   /* open output file */
   outfile=gzopen(outfname,"wb");
   if(!outfile){
      printf("Unable to open \"%s\" for writing.\n",outfname);
      exit(-1);
   }
   
   /* initialize for first screen update */
   re.event=0;

   /* Loop over input files */
   for(k=2;k<narg;k++){
   
      /* open input file */
      printf("\n");
      psim_readfile_init(argv[k]);
      end_of_file=0;
   
      /* read in events */
      while(!quit_program){
         
         /* update screen */
         if(time(NULL)!=last_time){
            printf("  %d  (%dM generated)  \r"
               ,Nevents
               ,Nevents_generated/1000000);
            fflush(stdout);
            last_time=time(NULL);
         }

         /* read in first event to add */
         if(!read_event(&re_tot))break;

         Nevents++;

			if(re_tot.event>last_event)
				Nevents_generated+=(re_tot.event-last_event);
			else
				Nevents_generated+=re_tot.event;

			last_event=re_tot.event;
         
         /* read in next N-1 events and add them to first */
         for(i=1;i<N_events_to_add;i++){
            if(!read_event(&re)){
               end_of_file=1;
               break;
            }
      
            Nevents++;

				if(re.event>last_event)
					Nevents_generated+=(re.event-last_event);
				else
					Nevents_generated+=re.event;

            last_event=re.event;
            
      
            AddEvent(&re,&re_tot);
         }
         
         /* break before writing added event if less than N */
         /* events are left available in this file.         */
         if(end_of_file)break;
      
         /* Write added event to output file */
         gzwrite(outfile,&re_tot,sizeof(raw_event_t));
         Nevents_written++;
      }
      
      /* close this input file */
      psim_readfile_close();
      printf("\n");
   }
   
   gzclose(outfile);
   printf("                             \n\n");
   printf(" %d events written \n",Nevents_written);
   printf(" %d events processed total\n",Nevents);
   printf(" %ld events generated total\n",Nevents_generated);
   printf("done.\n");
   
   return 0;
}

void AddEvent(raw_event_t *re1,raw_event_t *re2)
{
   int i,j,k;
   float t1,t2;
   part_info_t *pinfo;
   
   /* add energy depositions in calorimeter */
   for(i=0;i<2048;i++){
      re2->SHWDET[i] += re1->SHWDET[i];
      
      /* keep earliest time */
      t1=re1->TSHWDET[i];
      t2=re2->TSHWDET[i];
      
      if(t2=0.0 || (t1>0.0 && t1<t2))t2=t1;
      re1->TSHWDET[i]=t2;
   }
   
   /* keep most energetic part_info structures */
   for(i=0;i<re1->Npart_infos;i++){
   pinfo=&re1->part_info[i];
      
      for(j=0;j<re2->Npart_infos;j++)
         if(pinfo->E > re2->part_info[j].E)break;
      
      if(j<4){
         for(k=3;k>j;k--)re2->part_info[k]=re2->part_info[k-1];
         re2->part_info[j]=*pinfo;
         re2->Npart_infos++;
         if(re2->Npart_infos>3)re2->Npart_infos=4;
      }
   }
   
   re2->event=re1->event;
   re2->Edeposited+=re1->Edeposited;
   
}


void ParseArgs(int narg,char *argv[])
{
   int i;
   char *ptr;
   
   if(narg<3)Usage();
   
   N_events_to_add=atoi(argv[1]);
   
   strcpy(fname,argv[2]);
   strcpy(outfname,fname);
   
   ptr=strstr(outfname,".gz");
   if(ptr==NULL)ptr=&outfname[strlen(outfname)];
   sprintf(ptr,"_add%d.gz",N_events_to_add);
   
}

void Usage(void)
{
   printf("Usage:\n\tpsim_addevents # file.gz\n");
   printf("\n add energy deposition of # events and write it as a single\n");
   printf("event to the output file. Output filename will be the same as\n");
   printf("the input filename but with _add# inserted,\n\n");
   
   exit(-1);
}



