

/***********************************************************************/
/* psim_book.c
/*
/* D. Lawrence  4/13/99
/*
/* This simple program reads events from a "raw" data file compressed
/* with gzip and creates an output histogram file. It links to the
/* same routines used in psim:
/*
/* book_histos()
/* fill_histos()
/* close_histos()
/*
/***********************************************************************/



#include "psim.h"


#define Ethresh 0.0


int pawc_[NWPAWC];
int gcbank_[NWGEAN];
int quest_[100];


char fname[256],outfile[256]="";
FILE *gzfile;
int N_events_to_book=999999999;
int N_events_between_pauses=999999999;

extern GEOMETRY_TYPE;

/*--- global variables required by various objects linked to this program ---*/
int debug=0;
trig_rate_t trig_rate;
part_info_t part_info[max_part_infos];
int Npart_infos=0;
int nt1=100,nt2=10,nt3=11,nt4=12,nt5=13;
int flags[NUMFLAGS];
long N_GOOD_EVENT=0L;
float initial_photon_energy;
float max_energy;
/*--------------------------------------------------------------------------*/

int zero=0,one=1,two=2,three=3,four=4,five=5,six=6;
int seven=7,eight=8,nine=9,ten=10;
float fzero=0.,fone=1.,ftwo=2.,fthree=3.,ffour=4.,ffive=5.;
float fsix=6.,fseven=7.,feight=8.,fnine=9.,ften=10.;



int Nevents=0;
int Ntrig1=0,Ntrig2=0,Ntrig3=0,Ntrig4=0,Ntrig5=0,Ngood=0;
int Ntrig1g=0,Ntrig2g=0,Ntrig3g=0,Ntrig4g=0,Ntrig5g=0;
long Ngenerated;
float eff1=0.0,eff2=0.0,eff3=0.0,eff4=0.0,eff5=0.0;



void ParseArgs(int narg,char *argv[]);
int psim_readfile_init(char *fname);
int read_event(raw_event_t *re);
void EHandler(int sig);
void Print_Usage(void);
void PrintStatus(void);

int main(int narg,char *argv[])
{
   raw_event_t re;
   int i,j;
   time_t last_time=0L;
   part_info_t *p;
   char *ptr;
   int N;
   
   /*Try and exit gracefully should any errors occur*/
   signal(SIGPIPE,EHandler);
   signal(SIGTERM,EHandler);
   signal(SIGINT,EHandler);
   signal(SIGFPE,EHandler);

   ParseArgs(narg,argv);

   flags[fill_histograms]=1;
   flags[update_goods]=0;
   
   /* open input file */
   psim_readfile_init(fname);
   
   /* open output file */
   N=NWGEAN;
   gzebra_(&N);
   N=-NWPAWC;
   quest_[9] = 65000;
   if(flags[global_section])
      hlimap_(&N,"PRIM",strlen("PRIM"));
   else
      hlimit_(&N);
      
   
   if(strlen(outfile)==0){
      strcpy(outfile,fname);
      if(ptr=strstr(outfile,".gz"))
         strcpy(ptr,".hbook");
      else
         strcat(outfile,".hbook");
   }
   
   if(hropen(1,"LUN1",outfile,"N",1024,0)){
      printf("\nError opening \"%s\" for output!\n\n",outfile);
      return -1;
   }else{
      printf("\nOpened \"%s\" for output.\n",outfile);
   }
   hcdir_("//LUN1"," ",strlen("//LUN1"),strlen(" "));
   
   /* book histograms */
   announce_subsets=0;
   book_histos();

   printf("\n\n\n");
   
   /* read in events */
   while(read_event(&re)){
      if((++Nevents)%100 ==0){
         Ngenerated=re.event;   
         PrintStatus();
         ansi_up(10);
      }
      if(Nevents>=N_events_to_book)flags[quit]=1;
      
      fill_histos(&re);
      
      Ntrig1+=trig_rate.trig;
      Ntrig2+=trig_rate.trig2;
      Ntrig3+=trig_rate.trig3;
      Ntrig4+=trig_rate.trig4;
      Ntrig5+=trig_rate.trig5;
      
      if(trig_rate.good){
         Ntrig1g+=trig_rate.trig;
         Ntrig2g+=trig_rate.trig2;
         Ntrig3g+=trig_rate.trig3;
         Ntrig4g+=trig_rate.trig4;
         Ntrig5g+=trig_rate.trig5;
         Ngood++;
      }
      
      if((Nevents%N_events_between_pauses)==0){
         ansi_down(12);
         printf("Paused please hit <return> to continue\n");fflush(stdout);
         getc(stdin);
      }
            
      if(flags[quit])break;
   }

   ansi_down(12);
   PrintStatus();

   close_histos();

   
   
   printf("\ndone.\n");
   
   return 0;
}


void ParseArgs(int narg,char *argv[])
{
   int i;
   
   /* make sure all flags are reset (these are mostly unused by psim_book() */
   for(i=0;i<NUMFLAGS;i++)flags[i]=0;

   *fname=0;

   for(i=1;i<narg;i++){
      if(!strncmp(argv[i],"-N",2)){
         N_events_to_book=atoi(&argv[i][2]);
      }else if(!strncmp(argv[i],"-p",2)){
         if(strlen(argv[i])>2)
            N_events_between_pauses=atoi(&argv[i][2]);
         else
            N_events_between_pauses=1;
         if(N_events_between_pauses<=0)N_events_between_pauses=1;
      }else if(!strcmp(argv[i],"-gs")){
         flags[global_section]=1;
      }else if(!strncmp(argv[i],"-T",2)){
         trig_thresh=atof(&argv[i][2]);
      }else if(!strncmp(argv[i],"-G",2)){
         GEOMETRY_TYPE=atoi(&argv[i][2]);
      }else if(!strncmp(argv[i],"-o",2)){
         strcpy(outfile,&argv[i][2]);
      }else if(!strcmp(argv[i],"-h")){
         Print_Usage();
      }else{
         strcpy(fname,argv[i]);
      }
   }

   if(strlen(fname)==0){
      printf("You must provide a filename!\n");
      exit(-1);
   }
   
   printf("\ntrig_thresh set to %f GeV\n\n",trig_thresh);

}



void EHandler(int sig)
{
   flags[quit]=1;
   printf("\nReport: signal received of type ");
   if(sig==SIGPIPE)printf("SIGPIPE\n");
   if(sig==SIGTERM)printf("SIGTERM\n");
   if(sig==SIGINT)printf("SIGINT\n");
   if(sig==SIGFPE){
      printf("SIGFPE (event discarded)\n");
      flags[quit]=0;
      flags[bad_event]=1;
   }
  
}


void Print_Usage(void)
{
   printf("\nUsage:\n\tpsim_book [-N#] [-h] [-p[#]] [-gs] filename\n\n");
   
   printf("Book histograms from PrimEx simulation data.\n\n");
   
   printf("\t\t -N# Only book # events\n");
   printf("\t\t -p# Pause after # events (default is 1)\n");
   printf("\t\t -gs Map hbook to global section \"PRIM\"\n");
   printf("\t\t -Tthreshold set minimum photon energy for trigger\n");
   printf("\t\t -G# set geometry type (0=pi0 1=eta electron 2=eta photon)\n");
   printf("\t\t -oOutputfile set name of output file\n");
   printf("\t\t -h  Print this message\n");
   
   printf("\n");
   
   exit(0);

}



void PrintStatus(void)
{

   if(Ngood){
      eff1=100.0*(float)Ntrig1g/(float)Ngood;
      eff2=100.0*(float)Ntrig2g/(float)Ngood;
      eff3=100.0*(float)Ntrig3g/(float)Ngood;
      eff4=100.0*(float)Ntrig4g/(float)Ngood;
      eff5=100.0*(float)Ntrig5g/(float)Ngood;
   }
   
   printf(" %9d events processed  \n",Nevents);
   printf(" %9ld events generated  \n\n",Ngenerated);
   ansi_bold();
   printf("passed\t &good\t test\t efficiency \n");
   ansi_normal();
   printf(" %d\t %d\t good  \t %4.1f%% \n",Ngood,Ngood,100.0);
   printf(" %d\t %d\t trig1 \t %4.1f%% \n",Ntrig1,Ntrig1g,eff1);
   printf(" %d\t %d\t trig2 \t %4.1f%% \n",Ntrig2,Ntrig2g,eff2);
   printf(" %d\t %d\t trig3 \t %4.1f%% \n",Ntrig3,Ntrig3g,eff3);
   printf(" %d\t %d\t trig4 \t %4.1f%% \n",Ntrig4,Ntrig4g,eff4);
   printf(" %d\t %d\t trig5 \t %4.1f%% \n",Ntrig5,Ntrig5g,eff5);
   
   fflush(stdout);
}








