




#include "psim.h"


#define Ethresh 0.0


typedef struct {
   int   index;
   float limit;
}cut_t;
cut_t cut_lt[100];
cut_t cut_gt[100];
cut_t cut_eq[100];
int N_cut_lt=0,N_cut_gt=0,N_cut_eq=0;


char outfname[256]="",infname[128][256];
int Ninfiles=0;
FILE *outfile;
int quit_program=0;
int need_to_check_triggers=0;
int check_trig1=0,check_trig2=0,check_trig3=0,check_trig4=0,check_trig5=0;
int WRITE_SPARSE=0;
block_hit_info_t BHI[1024]; /* better not be more than 1024 detectors hit in an event!*/

extern char HEADER[sizeof(raw_event_t)];

char val_types[128][20]={"event","type","Ephoton","Edeposited",
      				"initialE","initialPx","initialPy","initialPz",
                  "initialR","initialX","initialY","initialZ",
                  "X1","Y1","Z1","E1","p1","T1","type1",
                  "X2","Y2","Z2","E2","p2","T2","type2",
                  "Npart","good","trig1","trig2","trig3","trig4",
                  "trig5","vaw","vaw_frac"
                  ""};


int psim_readfile_init(char *fname);
void psim_readfile_close(void);
int read_event(raw_event_t *re);

void ParseArgs(int narg,char *argv[]);
void AddCut(char *cutstr);
void PrintCutSummary(char *str,cut_t *c,int N_cut);
void Write_Header(int narg,char *argv[]);
void Usage(void);
void EHandler(int sig);


/*-------- Declarations to allow linking -------*/
   int nt1=100;
   int debug=0;
/*----------------------------------------------*/


float farray[256];

int main(int narg,char *argv[])
{
   raw_event_t re;
   int i,j,N;
   int Nevents=0,Nevents_written=0;
   time_t last_time=0L;
   part_info_t *p;
   int cut_event;
   float a;
   int good,trig1,trig2,trig3,trig4,trig5;
   int wrote_header=0;
   int Nhits;
   block_hit_info_t *bhi;
   char *ptr;
   
   /*Try and exit gracefully should any errors occur*/
   signal(SIGPIPE,EHandler);
   signal(SIGTERM,EHandler);
   signal(SIGINT,EHandler);
   signal(SIGFPE,EHandler);

   ParseArgs(narg,argv);

   /* initialize trigger simulators */
   announce_subsets=0;
   if(check_trig1 || check_trig5)init_trigger();
   if(check_trig2)init_trigger2();
   if(check_trig3)init_trigger3();
   if(check_trig4)init_trigger4();
   if(check_trig5)init_trigger5();

   /* print summary tables of cuts */
   PrintCutSummary("less than",cut_lt,N_cut_lt);
   PrintCutSummary("greater than",cut_gt,N_cut_gt);
   PrintCutSummary("equals",cut_eq,N_cut_eq);
   
   /* open output file */
   outfile=gzopen(outfname,"wb");
   if(!outfile){
      printf("Unable to open \"%s\" for writing\n",outfname);
      exit(-1);
   }
   
   
   /* loop over input files*/
   for(N=0;N<Ninfiles;N++){
   
      /* open input file */
      psim_readfile_init(infname[N]);
      if(!wrote_header){
			wrote_header=1;
			Write_Header(narg,argv);
      }

      /* read in events */
      while(read_event(&re)){
         if(quit_program)break;
      
         cut_event=0;
      
         /* Update screen */
         if(time(NULL)!=last_time){
            a=100.0*(float)Nevents_written/(float)Nevents;
            printf("  %d / %d  (%4.1f%%)\r"
               ,Nevents_written,Nevents,a);fflush(stdout);
            last_time=time(NULL);
         }
         Nevents++;
         
         /* check for triggers */
         if(check_trig1 || check_trig2 || check_trig3 ||check_trig4 || check_trig5)
            ProcessSubsets(re.SHWDET,re.TSHWDET);
         if(check_trig1)trig1=trigger(trig_thresh,NULL,NULL);
         if(check_trig2)trig2=trigger2(trig_thresh,NULL,NULL);
         if(check_trig3)trig3=trigger3(trig_thresh);
         if(check_trig4)trig4=trigger4(trig_thresh);
         if(check_trig5)trig5=trigger5(trig_thresh,NULL,NULL);
      
         /* fill floating point array */
         farray[ 0] = (float)re.event;
         farray[ 1] = (float)re.type;
         farray[ 2] = (float)re.Ephoton;
         farray[ 3] = (float)re.Edeposited;
         farray[ 4] = (float)re.initial_4vect.E;
         farray[ 5] = (float)re.initial_4vect.x;
         farray[ 6] = (float)re.initial_4vect.y;
         farray[ 7] = (float)re.initial_4vect.z;
         farray[ 8] = (float)re.initial_position.E;
         farray[ 9] = (float)re.initial_position.x;
         farray[10] = (float)re.initial_position.y;
         farray[11] = (float)re.initial_position.z;
         farray[12] = (float)re.part_info[0].X;
         farray[13] = (float)re.part_info[0].Y;
         farray[14] = (float)re.part_info[0].Z;
         farray[15] = (float)re.part_info[0].E;
         farray[16] = (float)re.part_info[0].p;
         farray[17] = (float)re.part_info[0].T;
         farray[18] = (float)re.part_info[0].type;
         farray[19] = (float)re.part_info[1].X;
         farray[20] = (float)re.part_info[1].Y;
         farray[21] = (float)re.part_info[1].Z;
         farray[22] = (float)re.part_info[1].E;
         farray[23] = (float)re.part_info[1].p;
         farray[24] = (float)re.part_info[1].T;
         farray[25] = (float)re.part_info[1].type;
         farray[26] = (float)re.Npart_infos;
         farray[27] = (float)good_event(&re);
         farray[28] = (float)trig1;
         farray[29] = (float)trig2;
         farray[30] = (float)trig3;
         farray[31] = (float)trig4;
         farray[32] = (float)trig5;
         farray[33] = (float)re.SHWDET[2040];
         farray[33] = (float)re.SHWDET[2040]/re.Ephoton;

         /*------- Apply cuts ---------*/
         for(i=0;i<N_cut_lt;i++){ /* less than cuts */
            if( farray[cut_lt[i].index] >= cut_lt[i].limit ){
               cut_event=1;
               break;
            }
         }
         if(cut_event)continue;

         for(i=0;i<N_cut_gt;i++){ /* greater than cuts */
            if( farray[cut_gt[i].index] <= cut_gt[i].limit ){
               cut_event=1;
               break;
            }
         }
         if(cut_event)continue;

         for(i=0;i<N_cut_eq;i++){ /* equals cuts */
            if( farray[cut_eq[i].index] != cut_eq[i].limit ){
               cut_event=1;
               break;
            }
         }
         if(cut_event)continue;
      
         /* Write event to output file */
   		if(WRITE_SPARSE){

				/* Get number of channels hit and copy values into global BHI structure */
				Nhits=0;
				for(i=0;i<2048;i++){
					if(re.SHWDET[i]>0.0){
						BHI[Nhits].index=i;
						BHI[Nhits].E=re.SHWDET[i];
						BHI[Nhits++].t=re.TSHWDET[i];
					}
				}

				((int*)re.SHWDET)[0]=Nhits;
				bhi=(block_hit_info_t*)&((int*)re.SHWDET)[1];
				for(i=0;i<Nhits;i++){
					bhi[i]=BHI[i];
				}

				/* Copy last half of raw_event_t structure into proper place */
				ptr=(char*)&re.TSHWDET[2048+1];
				memcpy((char*)&bhi[Nhits],ptr,sizeof(raw_event_t)-((int)ptr-(int)&re));

				/* Write sparse event */
				i = sizeof(raw_event_t)
					-(sizeof(float)*2048*2)
					+sizeof(int)
					+(Nhits*sizeof(block_hit_info_t));
				gzwrite(outfile,&re,i);

				/*
				printf("wrote event with:\n");
				printf("sizeof(raw_event_t)=%d\n",sizeof(raw_event_t));
				printf("i=%d\n",i);
				printf("Nhits=%d\n",Nhits);
				quit_program=1;
				*/
			}else{
         	gzwrite(outfile,&re,sizeof(raw_event_t));
			}
         Nevents_written++;
      }
      psim_readfile_close();
      printf("\n");
      if(quit_program)break;
   }
   gzclose(outfile);

   printf("                             \n\n");
   printf(" %d events kept\n",Nevents_written);
   printf(" %d events processed total\n",Nevents);
   printf("done.\n");
   
   return 0;
}


void ParseArgs(int narg,char *argv[])
{
   int i;
   char *ptr;
   
   if(narg<2)Usage();
   
   for(i=1;i<narg;i++){
      if(!strcmp(argv[i],"-h"))Usage();
      if(!strncmp(argv[i],"-o",2))strcpy(outfname,&argv[i][2]);
      
      if(!strncmp(argv[i],"-c",2))AddCut(&argv[i][2]);
      
      if(!strncmp(argv[i],"-T",2))trig_thresh=atof(&argv[i][2]);

      if(!strncmp(argv[i],"-s",2))WRITE_SPARSE=1;
      
      if(argv[i][0]!='-')strcpy(infname[Ninfiles++],argv[i]);
   }
   
   if(Ninfiles<1){
      printf("You must provide at least one input filename.\n");
      exit(-1);
   }
   
   if(strlen(outfname)==0){
      strcpy(outfname,infname[0]);
      ptr=strstr(outfname,".gz");
      if(ptr!=NULL){
         sprintf(ptr,"_filtered.gz");
      }else{
         strcat(outfname,"_filtered.gz");
      }
   }
   
   if(N_cut_lt){
      printf("\nless than cuts:\n\tindex\tlimit\n");
      for(i=0;i<N_cut_lt;i++)
         printf("\t%d\t%5.4g\t%s\n"
            ,cut_lt[i].index
            ,cut_lt[i].limit
            ,val_types[cut_lt[i].index]);
   }
   
   if(N_cut_gt){
      printf("\ngreater than cuts:\n\tindex\tlimit\n");
      for(i=0;i<N_cut_gt;i++)
         printf("\t%d\t%5.4g\t%s\n"
            ,cut_gt[i].index
            ,cut_gt[i].limit
            ,val_types[cut_gt[i].index]);
   }
   
   if(N_cut_eq){
      printf("\nequals cuts:\n\tindex\tlimit\n");
      for(i=0;i<N_cut_eq;i++)
         printf("\t%d\t%5.4g\t%s\n"
            ,cut_eq[i].index
            ,cut_eq[i].limit
            ,val_types[cut_eq[i].index]);
   }
   
   printf("\ntrig_thresh set to %f GeV\n\n",trig_thresh);
}

void AddCut(char *cutstr)
{
   char val[128],op[32],limit[128];
   int i;
   cut_t *cptr;
   int *Ncptr;
   
   bzero(val,128);
   bzero(op,32);
   bzero(limit,128);
   
   /* find "val" string */
   for(i=0;i<strlen(cutstr);i++){
      if(cutstr[i]=='.')break;
      val[strlen(val)]=cutstr[i];
   }

   /* find "op" string */
   for(i++;i<strlen(cutstr);i++){
      if(cutstr[i]=='.')break;
      op[strlen(op)]=cutstr[i];
   }

   /* find "limit" string */
   for(i++;i<strlen(cutstr);i++){
      limit[strlen(limit)]=cutstr[i];
   }

   /* determine "op" type */
   if(!strcmp(op,"lt")){
      cptr=cut_lt;
      Ncptr=&N_cut_lt;
   }else if(!strcmp(op,"gt")){
      cptr=cut_gt;
      Ncptr=&N_cut_gt;   
   }else if(!strcmp(op,"eq")){
      cptr=cut_eq;
      Ncptr=&N_cut_eq;
   }else{
      printf("\n\n Unknown op \"%s\".\n\n",op);
      exit(-1);
   }

   /* determine "val" type */
   i=-1;
   while(strlen(val_types[++i])){
      if(!strcmp(val,val_types[i])){
         cptr[*Ncptr].index=i;
         break;
      }
   }
   if(strlen(val_types[i])==0){
      printf("\n\n Unknown val \"%s\".\n\n",val);
      exit(-1);
   }
   
   if(!strcmp(val,"trig1"))check_trig1=1;
   if(!strcmp(val,"trig2"))check_trig2=1;
   if(!strcmp(val,"trig3"))check_trig3=1;
   if(!strcmp(val,"trig4"))check_trig4=1;
   if(!strcmp(val,"trig5"))check_trig5=1;
   
   /* determine limit */
   cptr[*Ncptr].limit=atof(limit);
   
   (*Ncptr)++;
}

void PrintCutSummary(char *str,cut_t *c,int N_cut)
{
   int i;
   
   if(N_cut==0)return;
   printf("\n%s cuts:\n\tindex\tlimit\n",str);
   for(i=0;i<N_cut;i++)
      printf("\t%d\t%5.4g\t%s\n"
            ,c[i].index
            ,c[i].limit
            ,val_types[c[i].index]);

}

void Write_Header(int narg,char *argv[])
{
	char str[8192];
	int i;
	
	strcpy(str,"PSIM -simulated data for PrimEx experiment\n\n");
	strcat(str,"This file produced by psim_filter with the following arguments:\n");
	for(i=0;i<narg;i++){strcat(str,argv[i]);strcat(str,"\n");}
	strcat(str,"\n===The header from the first file follows===\n");
	strcat(str,"===Headers from subsequent files were dropped===\n");
	gzwrite(outfile,str,strlen(str));
	gzwrite(outfile,HEADER,strlen(HEADER));
	sprintf(str,"data format:%s\n",WRITE_SPARSE ? "sparse":"full");
	gzwrite(outfile,str,strlen(str));
}

void Usage(void)
{
   int done=0,i,j;
   
   printf("\nUsage:\n\tpsim_filter [options] inputfile1 [inputfile2 ...]\n\n");
   printf("\t\t -oOutputFile \toutput filename\n");
   printf("\t\t -cCut \t\tspecify a cut (see below)\n");
   printf("\t\t -Tthreshold \tset minimum photon energy for \"good\" events\n");
   printf("\t\t -s \t\twrite output data in sparse format\n");
   printf("\t\t -h \t\tprint this message\n");
   printf("\n This small utility is intended to provided a quick and\n");
   printf("versitle way to create a file of selected events from another\n");
   printf("file. The format of the cut string is:\n");

   printf("\n\t\t val.op.limit\n\n");

   printf(" where \"val\" is one of:\n");
   i=-1;
   while(!done){
      printf("\n ");
      for(j=0;j<4;j++){
         if(strlen(val_types[++i])==0){done=1;break;}
         printf("%s \t%s"
            ,val_types[i]
            ,strlen(val_types[i])>5 ? " ":"\t ");
      }
   }
   
   printf("\n \"op\" is one of:\n\n");
   
   printf(" \t lt \t\t gt \t\t eq \n");
   
   printf("\n and \"limit\" is the numerical limit to apply.\n");
   printf("Examples:\n");
   printf("\t Edeposited.lt.1.4 \t (energy deposited is < 1.4 GeV)\n");
   printf("\t Npart.gt.2 \t\t (more than 2 particles hit detector)\n");
   printf("\t type.eq.3 \t\t (only events generator with generator type 3)\n");

   exit(0);
}

void EHandler(int sig)
{
   quit_program=1;
   printf("\n signal received of type ");
   if(sig==SIGPIPE)printf("SIGPIPE\n");
   if(sig==SIGTERM)printf("SIGTERM\n");
   if(sig==SIGINT)printf("SIGINT\n");
   if(sig==SIGFPE)printf("SIGFPE\n");
}

