




#include "psim.h"


int convert_endians=-1;
int SPARSE=0;
FILE *gzfile,*gzfile_raw;
long gzfile_size=0L;
char HEADER[sizeof(raw_event_t)];
block_hit_info_t BHI[1024]; /* better not be more than 1024 detectors hit in an event!*/
int GEOMETRY_TYPE=0;

int read_event(raw_event_t *re);
void ConvertEndian(raw_event_t *re);
void ReverseEndian(void *p,int n);

int psim_readfile_init(char *fname)
{
   raw_event_t re;
   int i,j;
   int Nevents;
   time_t last_time=0L;
   part_info_t *p;
   char *ptr;
   char str[256];
   
   /* open input file */
   gzfile_raw=fopen(fname,"rb");
   if(!gzfile_raw){
      printf("Unable to open \"%s\"\n",fname);
      exit(-1);
   }
   /* get file size */
   fseek(gzfile_raw,0,SEEK_END);
   gzfile_size=ftell(gzfile_raw);
   printf("filesize=%ld\n",gzfile_size);
   rewind(gzfile_raw);
   fclose(gzfile_raw);
   
   gzfile=gzopen(fname,"rb");
   if(!gzfile){
      printf("Unable to open \"%s\"\n",fname);
      exit(-1);
   }
   gzrewind(gzfile);
   printf("Events being read from \"%s\".\n",fname);
   fflush(stdout);

	/* read in header and store in global array */
	bzero(HEADER,sizeof(raw_event_t));
   gzread(gzfile,HEADER,sizeof(raw_event_t));
	gzrewind(gzfile);
	if(ptr=strstr(HEADER,"NEW EVENT:")){
		bzero(ptr,sizeof(raw_event_t)-((int)ptr-(int)HEADER));
		if(ptr=strstr(HEADER,"data format:")){
			if(sscanf(&ptr[strlen("data format:")],"%s",str)){
				if(!strcmp(str,"sparse"))SPARSE=1;
				if(!strcmp(str,"full"))SPARSE=0;
			}
		}
		if(ptr=strstr(HEADER,"Geometry type:")){
			if(sscanf(&ptr[strlen("Geometry type:")],"%s",str)){
				if(!strcmp(str,"pion"))GEOMETRY_TYPE=0;
				if(!strcmp(str,"eta_electron"))GEOMETRY_TYPE=1;
				if(!strcmp(str,"eta_photon"))GEOMETRY_TYPE=2;
				if(!strcmp(str,"unknown"))
					sscanf(&ptr[strlen("Geometry type: ")+strlen(str)]
						,"%d",&GEOMETRY_TYPE);				
			}
		}
	}   
   return 0;
}

void psim_readfile_close(void)
{
   gzclose(gzfile);
}

float psim_readfile_percent_done(void)
{
   return 100.0*(float)ftell(gzfile_raw)/(float)gzfile_size;

}

int read_event(raw_event_t *re)
{
   static char buff[sizeof(raw_event_t)*5];
   char *marker,*ptr;
   static int bytes_in_buff=0;
   int bytes_read=0;
   char event_header[]="NEW EVENT:\n";
   char event_trailer[]="END EVENT:\n";
	block_hit_info_t *bhi;
   int i,N;
   
   
   /* make sure an event header is in the buffer */
   if(bytes_in_buff<sizeof(raw_event_t)){
      bytes_read=gzread(gzfile,&buff[bytes_in_buff],sizeof(raw_event_t));
      bytes_in_buff+=bytes_read;
   }
   
   /* look for event header in buffer */
   marker=buff;
   for(i=0;i<bytes_in_buff;i++){
      if(!strncmp(marker++,event_header,sizeof(event_header)))break;
   }
   if(i>=bytes_in_buff)return 0;

   /* remove anything in buffer before event */
   bytes_in_buff-=(--marker)-buff;
   memmove(buff,marker,bytes_in_buff);

   /* make sure the whole event was read in */
   marker=buff;
   for(i=0;i<bytes_in_buff;i++){
      if(!strncmp(marker++,event_trailer,sizeof(event_trailer)))break;
   }
   if(i>=bytes_in_buff){
      bytes_read=gzread(gzfile,&buff[bytes_in_buff],sizeof(raw_event_t));
      bytes_in_buff+=bytes_read;

  		/* start searching a little back from where new data was read */
  		i-=strlen(event_trailer);
  		if(i<0)i=0;
  		marker=&buff[i];
   	for(i=i;i<bytes_in_buff;i++){
      	if(!strncmp(marker++,event_trailer,sizeof(event_trailer)))break;
   	}
   	if(i>=bytes_in_buff){
   		/*
   		printf("Couldn't find event trailer\n");
			printf("bytes_in_buff=%d\nstrlen(HEADER)=%d\n",bytes_in_buff,strlen(HEADER));
			printf("sizeof(raw_event_t)=%d\n",sizeof(raw_event_t));
   		*/
   		return 0;
   	}
   }

   /* copy event into calling routines data structure */
   memcpy(re,buff,sizeof(raw_event_t));

   /* remove this event from buffer */
   marker+=strlen(event_trailer);
   memmove(buff,marker,bytes_in_buff-(int)(marker-buff));
   bytes_in_buff-=(int)(marker-buff);

	/* rearrange for sparse data format if needed */
   if(SPARSE){

		/* Get number of channels hit and copy values into global BHI structure */
		N=((int*)re->SHWDET)[0];
		bhi=(block_hit_info_t*)&((int*)re->SHWDET)[1];
		for(i=0;i<N;i++){
			BHI[i]=bhi[i];
		}

		/* Copy last half of raw_event_t structure into proper place */
		ptr=(char*)&re->TSHWDET[2048+1];
		memcpy(ptr,(char*)&bhi[N],sizeof(raw_event_t)-((int)ptr-(int)re));

		/* Insert info for channels */
		bzero(re->SHWDET,2048*sizeof(float));
		bzero(re->TSHWDET,2048*sizeof(float));
		for(i=0;i<N;i++){
			re->SHWDET[BHI[i].index]=BHI[i].E;
			re->TSHWDET[BHI[i].index]=BHI[i].t;
		}		
	}

	/* Check first event only to see if endian must be converted */
	if(convert_endians<0){
		convert_endians = (re->Ephoton>20.0 || re->Ephoton<0.010) ? 1:0;
	}
	
   /* check if we need to change the "endian" style */
   if(convert_endians){
      /* In principle. for an event to have been written, Npart_infos */
      /* must have been 1 or greater.                                 */
      ConvertEndian(re);
   }
   
   return 1;
}


/**************************************************************/
/* converts big endian to little endian binaries in the event */
/* that the file was generated under a different architecture */
/* that this one.                                             */
/**************************************************************/
void ConvertEndian(raw_event_t *re)
{
   int i;
   vect4 *v;
   part_info_t *pinfo;

   ReverseEndian(&re->event,sizeof(long));
   
   for(i=0;i<2048;i++){
      ReverseEndian(&re->SHWDET[i],sizeof(float));   
      ReverseEndian(&re->TSHWDET[i],sizeof(float));   
   }
   
   ReverseEndian(&re->type        ,sizeof(int)   );
   ReverseEndian(&re->Ephoton     ,sizeof(float) );
   ReverseEndian(&re->Edeposited  ,sizeof(float) );
   
   v=&re->initial_4vect;
   ReverseEndian(&v->E  ,sizeof(float) );
   ReverseEndian(&v->x  ,sizeof(float) );
   ReverseEndian(&v->y  ,sizeof(float) );
   ReverseEndian(&v->z  ,sizeof(float) );

   v=&re->initial_position;
   ReverseEndian(&v->E  ,sizeof(float) );
   ReverseEndian(&v->x  ,sizeof(float) );
   ReverseEndian(&v->y  ,sizeof(float) );
   ReverseEndian(&v->z  ,sizeof(float) );

   for(i=0;i<max_part_infos;i++){
      pinfo=&re->part_info[i];
      ReverseEndian(&pinfo->X    ,sizeof(float) );
      ReverseEndian(&pinfo->Y    ,sizeof(float) );
      ReverseEndian(&pinfo->Z    ,sizeof(float) );
      ReverseEndian(&pinfo->E    ,sizeof(float) );
      ReverseEndian(&pinfo->p    ,sizeof(float) );
      ReverseEndian(&pinfo->T    ,sizeof(float) );
      ReverseEndian(&pinfo->type ,sizeof(int)   );
   }
   
   ReverseEndian(&re->Npart_infos ,sizeof(int)   );

   ReverseEndian(&re->subtype     ,sizeof(int)   );
   ReverseEndian(&re->decay_mech  ,sizeof(int)   );

}

void ReverseEndian(void *p,int n)
{
   unsigned char buff[256],*ptr=p;
   int i;
   
   memcpy(buff,p,n);
   for(i=0;i<n;i++){
      ptr[i]=buff[n-1-i];
   }
}





