#!/usr/bin/wish -f

global show_trig_group group_orientation
global continuous
set show_trig_group 1
set group_orientation 0
set continuous 0

proc Initialize {} {
   global show_trig1_groups show_trig2_groups
   global show_trig3_groups show_trig4_groups show_trig5_groups
   global continuous

   # top frame and canvas
   frame .c
   set len 750
   canvas .c.c -width $len -height $len -relief sunken -bg white -borderwidth 3
   pack .c.c -side left

   # frame beside the canvas
   frame .c.s
   pack .c.s -side right -fill both

   # Event Info
   set w .c.s.ei
   frame $w -borderwidth 3 -relief groove
   label $w.lab -text "Event Info"
   label $w.fil -text "File:" -width 30 -anchor w
   label $w.event -text "event:" -width 30 -anchor w
   label $w.geomtype -text "Geometry type:" -width 30 -anchor w
   label $w.type -text "MC type:" -width 30 -anchor w
   label $w.subtype -text "MC subtype:" -width 30 -anchor w
   label $w.decaymech -text "MC decaymech:" -width 30 -anchor w
   pack $w.lab $w.fil $w.event $w.geomtype $w.type $w.subtype $w.decaymech \
   	-side top -fill x
   pack $w -side top -fill both

   # trigger groups
   set w .c.s.tg
   frame $w -borderwidth 3 -relief groove
   label $w.lab -text "Trigger Info"
   label $w.good -text "good:" -anchor w
   radiobutton $w.t1 -text "trig1:" -anchor w -variable show_trig_group -value 1
   radiobutton $w.t2 -text "trig2:" -anchor w -variable show_trig_group -value 2
   radiobutton $w.t3 -text "trig3:" -anchor w -variable show_trig_group -value 3
   radiobutton $w.t4 -text "trig4:" -anchor w -variable show_trig_group -value 4
   radiobutton $w.t5 -text "trig5:" -anchor w -variable show_trig_group -value 5
   pack $w.lab $w.good $w.t1 $w.t2 $w.t3 $w.t4 $w.t5 -side top -fill both
   frame $w.go
   radiobutton $w.go.x -text "show X" -value 0 -variable group_orientation
   radiobutton $w.go.y -text "show Y" -value 1 -variable group_orientation
   pack $w.go.x $w.go.y -side left -fill x
   label $w.tb1x -text "trig1 x:" -width 25 -anchor w
   label $w.tb1y -text "trig1 y:" -width 25 -anchor w
   label $w.tb5xh -text "trig5 x hi:" -width 25 -anchor w
   label $w.tb5xl -text "trig5 x lo:" -width 25 -anchor w
   label $w.tb5yh -text "trig5 y hi:" -width 25 -anchor w
   label $w.tb5yl -text "trig5 y lo:" -width 25 -anchor w
   # The trig bits labels are packed/unpacked from C
   pack $w -side top -fill both

   # Group information
   set w .c.s.gi
   frame $w -borderwidth 3 -relief groove
   label $w.lab -text "Group Info"
   label $w.gn -text "Name:" -anchor w
   label $w.ge -text "Energy:" -anchor w -width 20
   label $w.ne -text "Nelements:" -anchor w -width 20
   frame $w.lbf
   listbox $w.lbf.lb -height 5 -yscrollcommand {.c.s.gi.lbf.sb set}
   scrollbar $w.lbf.sb -command {.c.s.gi.lbf.lb yview}
   pack $w.lbf.lb $w.lbf.sb -side right -fill both
   pack $w.lab $w.gn $w.ge $w.ne $w.lbf -side top -fill both
   pack $w -side top  -fill both

   # Bottom buttons
   frame .b
   button .b.quit -text "Quit" -command {destroy .}
   pack .b.quit -side right
   button .b.next -text "Next Event" -command {DrawEvent}
   pack .b.next -side left   
   button .b.cont -text "Continuous" -command {set continuous 1;DrawEvent}
   pack .b.cont -side left   
   button .b.stop -text "Stop" -command {set continuous 0}
   pack .b.stop -side left   
   
   # Cell Energy and time info
   frame .i
   label .i.energy -text "E:" -width 15 -borderwidth 2 -relief groove
   label .i.time -text "T:" -width 15 -borderwidth 2 -relief groove
   label .i.id -text "id:" -width 15 -borderwidth 2 -relief groove
   pack .i.energy .i.time .i.id -side left -fill both

   # Whole Detector Info
   set w .c.s.di
   frame $w -borderwidth 3 -relief groove
   label $w.edep -text "Edeposited:" -anchor w
   label $w.e1 -text "E1:" -width 15 -anchor w
   label $w.e2 -text "E2:" -width 15 -anchor w
   label $w.e3 -text "E3:" -width 15 -anchor w
   label $w.e4 -text "E4:" -width 15 -anchor w
   label $w.npart -text "Npart:" -width 15 -anchor w
   label $w.efrac -text "Fraction detected:" -width 15 -anchor w
   label $w.eleak -text "Energy leaked through:" -width 15 -anchor w
   label $w.eleakfrac -text "Fraction leaked through:" -width 15 -anchor w
   pack $w.eleakfrac $w.eleak $w.efrac $w.npart $w.e4 $w.e3 $w.e2 $w.e1 $w.edep \
      -side bottom -fill both
   pack $w -side top -fill both
   
   # Incident particle(s) info
   set w .c.s.ip
   frame $w -borderwidth 3 -relief groove
   label $w.lab -text "Pre-detected Particle Info"
   label $w.ephoton -text "Ephoton:" -anchor w
   pack $w.lab $w.ephoton -side top -fill both
   pack $w -side top -fill both
   

   pack .c .i .b -side top -padx 5 -pady 5 -fill both
}

proc DrawGrid {} {

	global geometry_type

	puts [format "geometry_type=%d\n" $geometry_type]
	
	if {[expr $geometry_type==0]} then {
   	for {set i 0} {[expr $i<27]} {incr i} {
      	for {set j 0} {[expr $j<27]} {incr j} {
         	DrawCell $i $j #FFF #000 grid 1 0
      	}
   	}

   	for {set i 0} {[expr $i<22]} {incr i} {
      	for {set j 0} {[expr $j<22]} {incr j} {
         	DrawCell $i $j #FFF #000 grid 1 1
      	}
   	}
   } elseif {[expr $geometry_type==1 || $geometry_type==2]} then {
   	for {set i 0} {[expr $i<35]} {incr i} {
      	for {set j 0} {[expr $j<35]} {incr j} {
         	DrawCell $i $j #FFF #000 grid 1 0
      	}
   	}
   }

}

proc DrawCell { i j fill outline tags width insertion} {

	global geometry_type

	if {[expr $geometry_type==0]} then {
   	set scale 24
   	set off   50
   } elseif {[expr $geometry_type==1 || $geometry_type==2]} then {
   	set scale 18
   	set off   50
   }

	set draw_it 0
	incr i
	incr j
   if {$insertion} then {
      if {[expr $i<=10 || $i>=13 || $j<=10 || $j>=13]} then {set draw_it 1}

      set xoff [expr ($scale*8.0)+$off]
      set yoff [expr ($scale*8.0)+$off]
      set xscale [expr $scale/2.0]
      set yscale [expr $scale/2.0]
   } else {
		if {[expr $geometry_type==0]} then {
      	if {[expr $i<=8 || $i>19 || $j<=8 || $j>19]} then {set draw_it 1}
   	} elseif {[expr $geometry_type==1]} then {
      	if {[expr $j>18 || $j<18 || $i>18 || $i<18]} then {set draw_it 1}
   	} elseif {[expr $geometry_type==2]} then {
      	if {[expr $j>18 || $j<18 || $i>18 || $i<0]} then {set draw_it 1}
		}
	}

	if {[expr $draw_it==0]} then {return}

   set xoff $off
   set yoff $off
   set xscale $scale
   set yscale $scale
   
   set x [expr $xoff+($xscale*$i)]
   set y [expr $yoff+($yscale*$j)]
   set x2 [expr $x+$xscale]
   set y2 [expr $y+$yscale]

   .c.c create rectangle $x $y $x2 $y2 \
      -fill $fill -outline $outline \
      -tags $tags -width $width
}

#Initialize
#DrawGrid

#tkwait window .


















